# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.lookup namespace
"""

import sys as _sys

from tensorflow._api.v2.compat.v2.lookup import experimental
from tensorflow.python.ops.lookup_ops import KeyValueTensorInitializer # line: 521
from tensorflow.python.ops.lookup_ops import StaticHashTable # line: 283
from tensorflow.python.ops.lookup_ops import StaticVocabularyTable # line: 1190
from tensorflow.python.ops.lookup_ops import TextFileIndex # line: 589
from tensorflow.python.ops.lookup_ops import TextFileInitializer # line: 609
