// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: xla/service/gpu/model/hlo_op_profile.proto
// Protobuf C++ Version: 5.28.3

#ifndef GOOGLE_PROTOBUF_INCLUDED_xla_2fservice_2fgpu_2fmodel_2fhlo_5fop_5fprofile_2eproto_2epb_2eh
#define GOOGLE_PROTOBUF_INCLUDED_xla_2fservice_2fgpu_2fmodel_2fhlo_5fop_5fprofile_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 5028003
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/map.h"  // IWYU pragma: export
#include "google/protobuf/map_entry.h"
#include "google/protobuf/map_field_inl.h"
#include "google/protobuf/unknown_field_set.h"
#include "xla/service/hlo.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_xla_2fservice_2fgpu_2fmodel_2fhlo_5fop_5fprofile_2eproto PROTOBUF_EXPORT

namespace google {
namespace protobuf {
namespace internal {
class AnyMetadata;
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct PROTOBUF_EXPORT TableStruct_xla_2fservice_2fgpu_2fmodel_2fhlo_5fop_5fprofile_2eproto {
  static const ::uint32_t offsets[];
};
PROTOBUF_EXPORT extern const ::google::protobuf::internal::DescriptorTable
    descriptor_table_xla_2fservice_2fgpu_2fmodel_2fhlo_5fop_5fprofile_2eproto;
namespace xla {
namespace gpu {
class DeviceHloInstructionProfiles;
struct DeviceHloInstructionProfilesDefaultTypeInternal;
PROTOBUF_EXPORT extern DeviceHloInstructionProfilesDefaultTypeInternal _DeviceHloInstructionProfiles_default_instance_;
class DeviceHloInstructionProfiles_EntriesEntry_DoNotUse;
struct DeviceHloInstructionProfiles_EntriesEntry_DoNotUseDefaultTypeInternal;
PROTOBUF_EXPORT extern DeviceHloInstructionProfiles_EntriesEntry_DoNotUseDefaultTypeInternal _DeviceHloInstructionProfiles_EntriesEntry_DoNotUse_default_instance_;
class HloInstructionProfile;
struct HloInstructionProfileDefaultTypeInternal;
PROTOBUF_EXPORT extern HloInstructionProfileDefaultTypeInternal _HloInstructionProfile_default_instance_;
class HloInstructionProfileList;
struct HloInstructionProfileListDefaultTypeInternal;
PROTOBUF_EXPORT extern HloInstructionProfileListDefaultTypeInternal _HloInstructionProfileList_default_instance_;
}  // namespace gpu
}  // namespace xla
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace xla {
namespace gpu {

// ===================================================================


// -------------------------------------------------------------------

class PROTOBUF_EXPORT HloInstructionProfile final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:xla.gpu.HloInstructionProfile) */ {
 public:
  inline HloInstructionProfile() : HloInstructionProfile(nullptr) {}
  ~HloInstructionProfile() PROTOBUF_FINAL;
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR HloInstructionProfile(
      ::google::protobuf::internal::ConstantInitialized);

  inline HloInstructionProfile(const HloInstructionProfile& from) : HloInstructionProfile(nullptr, from) {}
  inline HloInstructionProfile(HloInstructionProfile&& from) noexcept
      : HloInstructionProfile(nullptr, std::move(from)) {}
  inline HloInstructionProfile& operator=(const HloInstructionProfile& from) {
    CopyFrom(from);
    return *this;
  }
  inline HloInstructionProfile& operator=(HloInstructionProfile&& from) noexcept {
    if (this == &from) return *this;
    if (GetArena() == from.GetArena()
#ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetArena() != nullptr
#endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const HloInstructionProfile& default_instance() {
    return *internal_default_instance();
  }
  static inline const HloInstructionProfile* internal_default_instance() {
    return reinterpret_cast<const HloInstructionProfile*>(
        &_HloInstructionProfile_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(HloInstructionProfile& a, HloInstructionProfile& b) { a.Swap(&b); }
  inline void Swap(HloInstructionProfile* other) {
    if (other == this) return;
#ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() != nullptr && GetArena() == other->GetArena()) {
#else   // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() == other->GetArena()) {
#endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(HloInstructionProfile* other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  HloInstructionProfile* New(::google::protobuf::Arena* arena = nullptr) const PROTOBUF_FINAL {
    return ::google::protobuf::Message::DefaultConstruct<HloInstructionProfile>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const HloInstructionProfile& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const HloInstructionProfile& from) { HloInstructionProfile::MergeImpl(*this, from); }

  private:
  static void MergeImpl(
      ::google::protobuf::MessageLite& to_msg,
      const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* _InternalSerialize(
      const MessageLite& msg, ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* arena);
  void SharedDtor();
  void InternalSwap(HloInstructionProfile* other);
 private:
  friend class ::google::protobuf::internal::AnyMetadata;
  static ::absl::string_view FullMessageName() { return "xla.gpu.HloInstructionProfile"; }

 protected:
  explicit HloInstructionProfile(::google::protobuf::Arena* arena);
  HloInstructionProfile(::google::protobuf::Arena* arena, const HloInstructionProfile& from);
  HloInstructionProfile(::google::protobuf::Arena* arena, HloInstructionProfile&& from) noexcept
      : HloInstructionProfile(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;

 public:
  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kOperandsFieldNumber = 5,
    kFingerprintFieldNumber = 3,
    kInstructionFieldNumber = 1,
    kClockCyclesFieldNumber = 2,
    kFlopsFieldNumber = 4,
    kNetworkThroughputBytesPerSecFieldNumber = 6,
  };
  // repeated .xla.HloInstructionProto operands = 5;
  int operands_size() const;
  private:
  int _internal_operands_size() const;

  public:
  void clear_operands() ;
  ::xla::HloInstructionProto* mutable_operands(int index);
  ::google::protobuf::RepeatedPtrField<::xla::HloInstructionProto>* mutable_operands();

  private:
  const ::google::protobuf::RepeatedPtrField<::xla::HloInstructionProto>& _internal_operands() const;
  ::google::protobuf::RepeatedPtrField<::xla::HloInstructionProto>* _internal_mutable_operands();
  public:
  const ::xla::HloInstructionProto& operands(int index) const;
  ::xla::HloInstructionProto* add_operands();
  const ::google::protobuf::RepeatedPtrField<::xla::HloInstructionProto>& operands() const;
  // string fingerprint = 3;
  void clear_fingerprint() ;
  const std::string& fingerprint() const;
  template <typename Arg_ = const std::string&, typename... Args_>
  void set_fingerprint(Arg_&& arg, Args_... args);
  std::string* mutable_fingerprint();
  PROTOBUF_NODISCARD std::string* release_fingerprint();
  void set_allocated_fingerprint(std::string* value);

  private:
  const std::string& _internal_fingerprint() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_fingerprint(
      const std::string& value);
  std::string* _internal_mutable_fingerprint();

  public:
  // .xla.HloInstructionProto instruction = 1;
  bool has_instruction() const;
  void clear_instruction() ;
  const ::xla::HloInstructionProto& instruction() const;
  PROTOBUF_NODISCARD ::xla::HloInstructionProto* release_instruction();
  ::xla::HloInstructionProto* mutable_instruction();
  void set_allocated_instruction(::xla::HloInstructionProto* value);
  void unsafe_arena_set_allocated_instruction(::xla::HloInstructionProto* value);
  ::xla::HloInstructionProto* unsafe_arena_release_instruction();

  private:
  const ::xla::HloInstructionProto& _internal_instruction() const;
  ::xla::HloInstructionProto* _internal_mutable_instruction();

  public:
  // int64 clock_cycles = 2;
  void clear_clock_cycles() ;
  ::int64_t clock_cycles() const;
  void set_clock_cycles(::int64_t value);

  private:
  ::int64_t _internal_clock_cycles() const;
  void _internal_set_clock_cycles(::int64_t value);

  public:
  // int64 flops = 4;
  void clear_flops() ;
  ::int64_t flops() const;
  void set_flops(::int64_t value);

  private:
  ::int64_t _internal_flops() const;
  void _internal_set_flops(::int64_t value);

  public:
  // int64 network_throughput_bytes_per_sec = 6;
  void clear_network_throughput_bytes_per_sec() ;
  ::int64_t network_throughput_bytes_per_sec() const;
  void set_network_throughput_bytes_per_sec(::int64_t value);

  private:
  ::int64_t _internal_network_throughput_bytes_per_sec() const;
  void _internal_set_network_throughput_bytes_per_sec(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:xla.gpu.HloInstructionProfile)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      3, 6, 2,
      49, 2>
      _table_;


  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(
        ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena);
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena, const Impl_& from,
                          const HloInstructionProfile& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    mutable ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::xla::HloInstructionProto > operands_;
    ::google::protobuf::internal::ArenaStringPtr fingerprint_;
    ::xla::HloInstructionProto* instruction_;
    ::int64_t clock_cycles_;
    ::int64_t flops_;
    ::int64_t network_throughput_bytes_per_sec_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fservice_2fgpu_2fmodel_2fhlo_5fop_5fprofile_2eproto;
};
// -------------------------------------------------------------------

class PROTOBUF_EXPORT HloInstructionProfileList final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:xla.gpu.HloInstructionProfileList) */ {
 public:
  inline HloInstructionProfileList() : HloInstructionProfileList(nullptr) {}
  ~HloInstructionProfileList() PROTOBUF_FINAL;
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR HloInstructionProfileList(
      ::google::protobuf::internal::ConstantInitialized);

  inline HloInstructionProfileList(const HloInstructionProfileList& from) : HloInstructionProfileList(nullptr, from) {}
  inline HloInstructionProfileList(HloInstructionProfileList&& from) noexcept
      : HloInstructionProfileList(nullptr, std::move(from)) {}
  inline HloInstructionProfileList& operator=(const HloInstructionProfileList& from) {
    CopyFrom(from);
    return *this;
  }
  inline HloInstructionProfileList& operator=(HloInstructionProfileList&& from) noexcept {
    if (this == &from) return *this;
    if (GetArena() == from.GetArena()
#ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetArena() != nullptr
#endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const HloInstructionProfileList& default_instance() {
    return *internal_default_instance();
  }
  static inline const HloInstructionProfileList* internal_default_instance() {
    return reinterpret_cast<const HloInstructionProfileList*>(
        &_HloInstructionProfileList_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(HloInstructionProfileList& a, HloInstructionProfileList& b) { a.Swap(&b); }
  inline void Swap(HloInstructionProfileList* other) {
    if (other == this) return;
#ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() != nullptr && GetArena() == other->GetArena()) {
#else   // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() == other->GetArena()) {
#endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(HloInstructionProfileList* other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  HloInstructionProfileList* New(::google::protobuf::Arena* arena = nullptr) const PROTOBUF_FINAL {
    return ::google::protobuf::Message::DefaultConstruct<HloInstructionProfileList>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const HloInstructionProfileList& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const HloInstructionProfileList& from) { HloInstructionProfileList::MergeImpl(*this, from); }

  private:
  static void MergeImpl(
      ::google::protobuf::MessageLite& to_msg,
      const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* _InternalSerialize(
      const MessageLite& msg, ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* arena);
  void SharedDtor();
  void InternalSwap(HloInstructionProfileList* other);
 private:
  friend class ::google::protobuf::internal::AnyMetadata;
  static ::absl::string_view FullMessageName() { return "xla.gpu.HloInstructionProfileList"; }

 protected:
  explicit HloInstructionProfileList(::google::protobuf::Arena* arena);
  HloInstructionProfileList(::google::protobuf::Arena* arena, const HloInstructionProfileList& from);
  HloInstructionProfileList(::google::protobuf::Arena* arena, HloInstructionProfileList&& from) noexcept
      : HloInstructionProfileList(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;

 public:
  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kEntriesFieldNumber = 1,
  };
  // repeated .xla.gpu.HloInstructionProfile entries = 1;
  int entries_size() const;
  private:
  int _internal_entries_size() const;

  public:
  void clear_entries() ;
  ::xla::gpu::HloInstructionProfile* mutable_entries(int index);
  ::google::protobuf::RepeatedPtrField<::xla::gpu::HloInstructionProfile>* mutable_entries();

  private:
  const ::google::protobuf::RepeatedPtrField<::xla::gpu::HloInstructionProfile>& _internal_entries() const;
  ::google::protobuf::RepeatedPtrField<::xla::gpu::HloInstructionProfile>* _internal_mutable_entries();
  public:
  const ::xla::gpu::HloInstructionProfile& entries(int index) const;
  ::xla::gpu::HloInstructionProfile* add_entries();
  const ::google::protobuf::RepeatedPtrField<::xla::gpu::HloInstructionProfile>& entries() const;
  // @@protoc_insertion_point(class_scope:xla.gpu.HloInstructionProfileList)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      0, 1, 1,
      0, 2>
      _table_;


  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(
        ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena);
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena, const Impl_& from,
                          const HloInstructionProfileList& from_msg);
    ::google::protobuf::RepeatedPtrField< ::xla::gpu::HloInstructionProfile > entries_;
    mutable ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fservice_2fgpu_2fmodel_2fhlo_5fop_5fprofile_2eproto;
};
// -------------------------------------------------------------------

class DeviceHloInstructionProfiles_EntriesEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<
          DeviceHloInstructionProfiles_EntriesEntry_DoNotUse, std::string, ::xla::gpu::HloInstructionProfileList,
          ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
          ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE> {
 public:
  using SuperType = ::google::protobuf::internal::MapEntry<
      DeviceHloInstructionProfiles_EntriesEntry_DoNotUse, std::string, ::xla::gpu::HloInstructionProfileList,
      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
      ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE>;
  DeviceHloInstructionProfiles_EntriesEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR DeviceHloInstructionProfiles_EntriesEntry_DoNotUse(
      ::google::protobuf::internal::ConstantInitialized);
  explicit DeviceHloInstructionProfiles_EntriesEntry_DoNotUse(::google::protobuf::Arena* arena);
  static const DeviceHloInstructionProfiles_EntriesEntry_DoNotUse* internal_default_instance() {
    return reinterpret_cast<const DeviceHloInstructionProfiles_EntriesEntry_DoNotUse*>(
        &_DeviceHloInstructionProfiles_EntriesEntry_DoNotUse_default_instance_);
  }


 private:
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      1, 2, 1,
      61, 2>
      _table_;

  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;
  friend struct ::TableStruct_xla_2fservice_2fgpu_2fmodel_2fhlo_5fop_5fprofile_2eproto;
};
// -------------------------------------------------------------------

class PROTOBUF_EXPORT DeviceHloInstructionProfiles final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:xla.gpu.DeviceHloInstructionProfiles) */ {
 public:
  inline DeviceHloInstructionProfiles() : DeviceHloInstructionProfiles(nullptr) {}
  ~DeviceHloInstructionProfiles() PROTOBUF_FINAL;
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR DeviceHloInstructionProfiles(
      ::google::protobuf::internal::ConstantInitialized);

  inline DeviceHloInstructionProfiles(const DeviceHloInstructionProfiles& from) : DeviceHloInstructionProfiles(nullptr, from) {}
  inline DeviceHloInstructionProfiles(DeviceHloInstructionProfiles&& from) noexcept
      : DeviceHloInstructionProfiles(nullptr, std::move(from)) {}
  inline DeviceHloInstructionProfiles& operator=(const DeviceHloInstructionProfiles& from) {
    CopyFrom(from);
    return *this;
  }
  inline DeviceHloInstructionProfiles& operator=(DeviceHloInstructionProfiles&& from) noexcept {
    if (this == &from) return *this;
    if (GetArena() == from.GetArena()
#ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetArena() != nullptr
#endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const DeviceHloInstructionProfiles& default_instance() {
    return *internal_default_instance();
  }
  static inline const DeviceHloInstructionProfiles* internal_default_instance() {
    return reinterpret_cast<const DeviceHloInstructionProfiles*>(
        &_DeviceHloInstructionProfiles_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(DeviceHloInstructionProfiles& a, DeviceHloInstructionProfiles& b) { a.Swap(&b); }
  inline void Swap(DeviceHloInstructionProfiles* other) {
    if (other == this) return;
#ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() != nullptr && GetArena() == other->GetArena()) {
#else   // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() == other->GetArena()) {
#endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(DeviceHloInstructionProfiles* other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  DeviceHloInstructionProfiles* New(::google::protobuf::Arena* arena = nullptr) const PROTOBUF_FINAL {
    return ::google::protobuf::Message::DefaultConstruct<DeviceHloInstructionProfiles>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const DeviceHloInstructionProfiles& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const DeviceHloInstructionProfiles& from) { DeviceHloInstructionProfiles::MergeImpl(*this, from); }

  private:
  static void MergeImpl(
      ::google::protobuf::MessageLite& to_msg,
      const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* _InternalSerialize(
      const MessageLite& msg, ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* arena);
  void SharedDtor();
  void InternalSwap(DeviceHloInstructionProfiles* other);
 private:
  friend class ::google::protobuf::internal::AnyMetadata;
  static ::absl::string_view FullMessageName() { return "xla.gpu.DeviceHloInstructionProfiles"; }

 protected:
  explicit DeviceHloInstructionProfiles(::google::protobuf::Arena* arena);
  DeviceHloInstructionProfiles(::google::protobuf::Arena* arena, const DeviceHloInstructionProfiles& from);
  DeviceHloInstructionProfiles(::google::protobuf::Arena* arena, DeviceHloInstructionProfiles&& from) noexcept
      : DeviceHloInstructionProfiles(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;

 public:
  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kEntriesFieldNumber = 2,
  };
  // map<string, .xla.gpu.HloInstructionProfileList> entries = 2;
  int entries_size() const;
  private:
  int _internal_entries_size() const;

  public:
  void clear_entries() ;
  const ::google::protobuf::Map<std::string, ::xla::gpu::HloInstructionProfileList>& entries() const;
  ::google::protobuf::Map<std::string, ::xla::gpu::HloInstructionProfileList>* mutable_entries();

  private:
  const ::google::protobuf::Map<std::string, ::xla::gpu::HloInstructionProfileList>& _internal_entries() const;
  ::google::protobuf::Map<std::string, ::xla::gpu::HloInstructionProfileList>* _internal_mutable_entries();

  public:
  // @@protoc_insertion_point(class_scope:xla.gpu.DeviceHloInstructionProfiles)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      0, 1, 2,
      52, 2>
      _table_;


  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(
        ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena);
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena, const Impl_& from,
                          const DeviceHloInstructionProfiles& from_msg);
    ::google::protobuf::internal::MapField<DeviceHloInstructionProfiles_EntriesEntry_DoNotUse, std::string, ::xla::gpu::HloInstructionProfileList,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE>
        entries_;
    mutable ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fservice_2fgpu_2fmodel_2fhlo_5fop_5fprofile_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// HloInstructionProfile

// .xla.HloInstructionProto instruction = 1;
inline bool HloInstructionProfile::has_instruction() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.instruction_ != nullptr);
  return value;
}
inline const ::xla::HloInstructionProto& HloInstructionProfile::_internal_instruction() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::xla::HloInstructionProto* p = _impl_.instruction_;
  return p != nullptr ? *p : reinterpret_cast<const ::xla::HloInstructionProto&>(::xla::_HloInstructionProto_default_instance_);
}
inline const ::xla::HloInstructionProto& HloInstructionProfile::instruction() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:xla.gpu.HloInstructionProfile.instruction)
  return _internal_instruction();
}
inline void HloInstructionProfile::unsafe_arena_set_allocated_instruction(::xla::HloInstructionProto* value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.instruction_);
  }
  _impl_.instruction_ = reinterpret_cast<::xla::HloInstructionProto*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:xla.gpu.HloInstructionProfile.instruction)
}
inline ::xla::HloInstructionProto* HloInstructionProfile::release_instruction() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000001u;
  ::xla::HloInstructionProto* released = _impl_.instruction_;
  _impl_.instruction_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
  released = ::google::protobuf::internal::DuplicateIfNonNull(released);
  if (GetArena() == nullptr) {
    delete old;
  }
#else   // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArena() != nullptr) {
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return released;
}
inline ::xla::HloInstructionProto* HloInstructionProfile::unsafe_arena_release_instruction() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:xla.gpu.HloInstructionProfile.instruction)

  _impl_._has_bits_[0] &= ~0x00000001u;
  ::xla::HloInstructionProto* temp = _impl_.instruction_;
  _impl_.instruction_ = nullptr;
  return temp;
}
inline ::xla::HloInstructionProto* HloInstructionProfile::_internal_mutable_instruction() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.instruction_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::xla::HloInstructionProto>(GetArena());
    _impl_.instruction_ = reinterpret_cast<::xla::HloInstructionProto*>(p);
  }
  return _impl_.instruction_;
}
inline ::xla::HloInstructionProto* HloInstructionProfile::mutable_instruction() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000001u;
  ::xla::HloInstructionProto* _msg = _internal_mutable_instruction();
  // @@protoc_insertion_point(field_mutable:xla.gpu.HloInstructionProfile.instruction)
  return _msg;
}
inline void HloInstructionProfile::set_allocated_instruction(::xla::HloInstructionProto* value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.instruction_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::MessageLite*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }

  _impl_.instruction_ = reinterpret_cast<::xla::HloInstructionProto*>(value);
  // @@protoc_insertion_point(field_set_allocated:xla.gpu.HloInstructionProfile.instruction)
}

// repeated .xla.HloInstructionProto operands = 5;
inline int HloInstructionProfile::_internal_operands_size() const {
  return _internal_operands().size();
}
inline int HloInstructionProfile::operands_size() const {
  return _internal_operands_size();
}
inline ::xla::HloInstructionProto* HloInstructionProfile::mutable_operands(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:xla.gpu.HloInstructionProfile.operands)
  return _internal_mutable_operands()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::xla::HloInstructionProto>* HloInstructionProfile::mutable_operands()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:xla.gpu.HloInstructionProfile.operands)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_operands();
}
inline const ::xla::HloInstructionProto& HloInstructionProfile::operands(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:xla.gpu.HloInstructionProfile.operands)
  return _internal_operands().Get(index);
}
inline ::xla::HloInstructionProto* HloInstructionProfile::add_operands() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::xla::HloInstructionProto* _add = _internal_mutable_operands()->Add();
  // @@protoc_insertion_point(field_add:xla.gpu.HloInstructionProfile.operands)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::xla::HloInstructionProto>& HloInstructionProfile::operands() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:xla.gpu.HloInstructionProfile.operands)
  return _internal_operands();
}
inline const ::google::protobuf::RepeatedPtrField<::xla::HloInstructionProto>&
HloInstructionProfile::_internal_operands() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.operands_;
}
inline ::google::protobuf::RepeatedPtrField<::xla::HloInstructionProto>*
HloInstructionProfile::_internal_mutable_operands() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.operands_;
}

// int64 clock_cycles = 2;
inline void HloInstructionProfile::clear_clock_cycles() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.clock_cycles_ = ::int64_t{0};
}
inline ::int64_t HloInstructionProfile::clock_cycles() const {
  // @@protoc_insertion_point(field_get:xla.gpu.HloInstructionProfile.clock_cycles)
  return _internal_clock_cycles();
}
inline void HloInstructionProfile::set_clock_cycles(::int64_t value) {
  _internal_set_clock_cycles(value);
  // @@protoc_insertion_point(field_set:xla.gpu.HloInstructionProfile.clock_cycles)
}
inline ::int64_t HloInstructionProfile::_internal_clock_cycles() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.clock_cycles_;
}
inline void HloInstructionProfile::_internal_set_clock_cycles(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.clock_cycles_ = value;
}

// string fingerprint = 3;
inline void HloInstructionProfile::clear_fingerprint() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fingerprint_.ClearToEmpty();
}
inline const std::string& HloInstructionProfile::fingerprint() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:xla.gpu.HloInstructionProfile.fingerprint)
  return _internal_fingerprint();
}
template <typename Arg_, typename... Args_>
inline PROTOBUF_ALWAYS_INLINE void HloInstructionProfile::set_fingerprint(Arg_&& arg,
                                                     Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fingerprint_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:xla.gpu.HloInstructionProfile.fingerprint)
}
inline std::string* HloInstructionProfile::mutable_fingerprint() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  std::string* _s = _internal_mutable_fingerprint();
  // @@protoc_insertion_point(field_mutable:xla.gpu.HloInstructionProfile.fingerprint)
  return _s;
}
inline const std::string& HloInstructionProfile::_internal_fingerprint() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fingerprint_.Get();
}
inline void HloInstructionProfile::_internal_set_fingerprint(const std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fingerprint_.Set(value, GetArena());
}
inline std::string* HloInstructionProfile::_internal_mutable_fingerprint() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fingerprint_.Mutable( GetArena());
}
inline std::string* HloInstructionProfile::release_fingerprint() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:xla.gpu.HloInstructionProfile.fingerprint)
  return _impl_.fingerprint_.Release();
}
inline void HloInstructionProfile::set_allocated_fingerprint(std::string* value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fingerprint_.SetAllocated(value, GetArena());
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
        if (_impl_.fingerprint_.IsDefault()) {
          _impl_.fingerprint_.Set("", GetArena());
        }
  #endif  // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:xla.gpu.HloInstructionProfile.fingerprint)
}

// int64 flops = 4;
inline void HloInstructionProfile::clear_flops() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flops_ = ::int64_t{0};
}
inline ::int64_t HloInstructionProfile::flops() const {
  // @@protoc_insertion_point(field_get:xla.gpu.HloInstructionProfile.flops)
  return _internal_flops();
}
inline void HloInstructionProfile::set_flops(::int64_t value) {
  _internal_set_flops(value);
  // @@protoc_insertion_point(field_set:xla.gpu.HloInstructionProfile.flops)
}
inline ::int64_t HloInstructionProfile::_internal_flops() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flops_;
}
inline void HloInstructionProfile::_internal_set_flops(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flops_ = value;
}

// int64 network_throughput_bytes_per_sec = 6;
inline void HloInstructionProfile::clear_network_throughput_bytes_per_sec() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.network_throughput_bytes_per_sec_ = ::int64_t{0};
}
inline ::int64_t HloInstructionProfile::network_throughput_bytes_per_sec() const {
  // @@protoc_insertion_point(field_get:xla.gpu.HloInstructionProfile.network_throughput_bytes_per_sec)
  return _internal_network_throughput_bytes_per_sec();
}
inline void HloInstructionProfile::set_network_throughput_bytes_per_sec(::int64_t value) {
  _internal_set_network_throughput_bytes_per_sec(value);
  // @@protoc_insertion_point(field_set:xla.gpu.HloInstructionProfile.network_throughput_bytes_per_sec)
}
inline ::int64_t HloInstructionProfile::_internal_network_throughput_bytes_per_sec() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.network_throughput_bytes_per_sec_;
}
inline void HloInstructionProfile::_internal_set_network_throughput_bytes_per_sec(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.network_throughput_bytes_per_sec_ = value;
}

// -------------------------------------------------------------------

// HloInstructionProfileList

// repeated .xla.gpu.HloInstructionProfile entries = 1;
inline int HloInstructionProfileList::_internal_entries_size() const {
  return _internal_entries().size();
}
inline int HloInstructionProfileList::entries_size() const {
  return _internal_entries_size();
}
inline void HloInstructionProfileList::clear_entries() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.entries_.Clear();
}
inline ::xla::gpu::HloInstructionProfile* HloInstructionProfileList::mutable_entries(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:xla.gpu.HloInstructionProfileList.entries)
  return _internal_mutable_entries()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::xla::gpu::HloInstructionProfile>* HloInstructionProfileList::mutable_entries()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:xla.gpu.HloInstructionProfileList.entries)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_entries();
}
inline const ::xla::gpu::HloInstructionProfile& HloInstructionProfileList::entries(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:xla.gpu.HloInstructionProfileList.entries)
  return _internal_entries().Get(index);
}
inline ::xla::gpu::HloInstructionProfile* HloInstructionProfileList::add_entries() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::xla::gpu::HloInstructionProfile* _add = _internal_mutable_entries()->Add();
  // @@protoc_insertion_point(field_add:xla.gpu.HloInstructionProfileList.entries)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::xla::gpu::HloInstructionProfile>& HloInstructionProfileList::entries() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:xla.gpu.HloInstructionProfileList.entries)
  return _internal_entries();
}
inline const ::google::protobuf::RepeatedPtrField<::xla::gpu::HloInstructionProfile>&
HloInstructionProfileList::_internal_entries() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.entries_;
}
inline ::google::protobuf::RepeatedPtrField<::xla::gpu::HloInstructionProfile>*
HloInstructionProfileList::_internal_mutable_entries() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.entries_;
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// DeviceHloInstructionProfiles

// map<string, .xla.gpu.HloInstructionProfileList> entries = 2;
inline int DeviceHloInstructionProfiles::_internal_entries_size() const {
  return _internal_entries().size();
}
inline int DeviceHloInstructionProfiles::entries_size() const {
  return _internal_entries_size();
}
inline void DeviceHloInstructionProfiles::clear_entries() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.entries_.Clear();
}
inline const ::google::protobuf::Map<std::string, ::xla::gpu::HloInstructionProfileList>& DeviceHloInstructionProfiles::_internal_entries() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.entries_.GetMap();
}
inline const ::google::protobuf::Map<std::string, ::xla::gpu::HloInstructionProfileList>& DeviceHloInstructionProfiles::entries() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:xla.gpu.DeviceHloInstructionProfiles.entries)
  return _internal_entries();
}
inline ::google::protobuf::Map<std::string, ::xla::gpu::HloInstructionProfileList>* DeviceHloInstructionProfiles::_internal_mutable_entries() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.entries_.MutableMap();
}
inline ::google::protobuf::Map<std::string, ::xla::gpu::HloInstructionProfileList>* DeviceHloInstructionProfiles::mutable_entries() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_map:xla.gpu.DeviceHloInstructionProfiles.entries)
  return _internal_mutable_entries();
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace gpu
}  // namespace xla


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // GOOGLE_PROTOBUF_INCLUDED_xla_2fservice_2fgpu_2fmodel_2fhlo_5fop_5fprofile_2eproto_2epb_2eh
