// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: xla/service/buffer_assignment.proto
// Protobuf C++ Version: 5.28.3

#ifndef GOOGLE_PROTOBUF_INCLUDED_xla_2fservice_2fbuffer_5fassignment_2eproto_2epb_2eh
#define GOOGLE_PROTOBUF_INCLUDED_xla_2fservice_2fbuffer_5fassignment_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 5028003
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_xla_2fservice_2fbuffer_5fassignment_2eproto PROTOBUF_EXPORT

namespace google {
namespace protobuf {
namespace internal {
class AnyMetadata;
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct PROTOBUF_EXPORT TableStruct_xla_2fservice_2fbuffer_5fassignment_2eproto {
  static const ::uint32_t offsets[];
};
PROTOBUF_EXPORT extern const ::google::protobuf::internal::DescriptorTable
    descriptor_table_xla_2fservice_2fbuffer_5fassignment_2eproto;
namespace xla {
namespace buffer_assignment {
class BufferAllocationSliceProto;
struct BufferAllocationSliceProtoDefaultTypeInternal;
PROTOBUF_EXPORT extern BufferAllocationSliceProtoDefaultTypeInternal _BufferAllocationSliceProto_default_instance_;
class BufferIsolationConfig;
struct BufferIsolationConfigDefaultTypeInternal;
PROTOBUF_EXPORT extern BufferIsolationConfigDefaultTypeInternal _BufferIsolationConfig_default_instance_;
}  // namespace buffer_assignment
}  // namespace xla
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace xla {
namespace buffer_assignment {

// ===================================================================


// -------------------------------------------------------------------

class PROTOBUF_EXPORT BufferIsolationConfig final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:xla.buffer_assignment.BufferIsolationConfig) */ {
 public:
  inline BufferIsolationConfig() : BufferIsolationConfig(nullptr) {}
  ~BufferIsolationConfig() PROTOBUF_FINAL;
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR BufferIsolationConfig(
      ::google::protobuf::internal::ConstantInitialized);

  inline BufferIsolationConfig(const BufferIsolationConfig& from) : BufferIsolationConfig(nullptr, from) {}
  inline BufferIsolationConfig(BufferIsolationConfig&& from) noexcept
      : BufferIsolationConfig(nullptr, std::move(from)) {}
  inline BufferIsolationConfig& operator=(const BufferIsolationConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline BufferIsolationConfig& operator=(BufferIsolationConfig&& from) noexcept {
    if (this == &from) return *this;
    if (GetArena() == from.GetArena()
#ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetArena() != nullptr
#endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const BufferIsolationConfig& default_instance() {
    return *internal_default_instance();
  }
  static inline const BufferIsolationConfig* internal_default_instance() {
    return reinterpret_cast<const BufferIsolationConfig*>(
        &_BufferIsolationConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(BufferIsolationConfig& a, BufferIsolationConfig& b) { a.Swap(&b); }
  inline void Swap(BufferIsolationConfig* other) {
    if (other == this) return;
#ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() != nullptr && GetArena() == other->GetArena()) {
#else   // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() == other->GetArena()) {
#endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(BufferIsolationConfig* other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  BufferIsolationConfig* New(::google::protobuf::Arena* arena = nullptr) const PROTOBUF_FINAL {
    return ::google::protobuf::Message::DefaultConstruct<BufferIsolationConfig>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const BufferIsolationConfig& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const BufferIsolationConfig& from) { BufferIsolationConfig::MergeImpl(*this, from); }

  private:
  static void MergeImpl(
      ::google::protobuf::MessageLite& to_msg,
      const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* _InternalSerialize(
      const MessageLite& msg, ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* arena);
  void SharedDtor();
  void InternalSwap(BufferIsolationConfig* other);
 private:
  friend class ::google::protobuf::internal::AnyMetadata;
  static ::absl::string_view FullMessageName() { return "xla.buffer_assignment.BufferIsolationConfig"; }

 protected:
  explicit BufferIsolationConfig(::google::protobuf::Arena* arena);
  BufferIsolationConfig(::google::protobuf::Arena* arena, const BufferIsolationConfig& from);
  BufferIsolationConfig(::google::protobuf::Arena* arena, BufferIsolationConfig&& from) noexcept
      : BufferIsolationConfig(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;

 public:
  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kIsolationColorsFieldNumber = 5,
    kBaseOffsetBytesFieldNumber = 1,
    kIsolationFuelFieldNumber = 2,
    kIsolationPaddingBytesFieldNumber = 3,
    kIsolationOrderSaltFieldNumber = 4,
  };
  // repeated int32 isolation_colors = 5;
  int isolation_colors_size() const;
  private:
  int _internal_isolation_colors_size() const;

  public:
  void clear_isolation_colors() ;
  ::int32_t isolation_colors(int index) const;
  void set_isolation_colors(int index, ::int32_t value);
  void add_isolation_colors(::int32_t value);
  const ::google::protobuf::RepeatedField<::int32_t>& isolation_colors() const;
  ::google::protobuf::RepeatedField<::int32_t>* mutable_isolation_colors();

  private:
  const ::google::protobuf::RepeatedField<::int32_t>& _internal_isolation_colors() const;
  ::google::protobuf::RepeatedField<::int32_t>* _internal_mutable_isolation_colors();

  public:
  // int64 base_offset_bytes = 1;
  void clear_base_offset_bytes() ;
  ::int64_t base_offset_bytes() const;
  void set_base_offset_bytes(::int64_t value);

  private:
  ::int64_t _internal_base_offset_bytes() const;
  void _internal_set_base_offset_bytes(::int64_t value);

  public:
  // int64 isolation_fuel = 2;
  void clear_isolation_fuel() ;
  ::int64_t isolation_fuel() const;
  void set_isolation_fuel(::int64_t value);

  private:
  ::int64_t _internal_isolation_fuel() const;
  void _internal_set_isolation_fuel(::int64_t value);

  public:
  // int64 isolation_padding_bytes = 3;
  void clear_isolation_padding_bytes() ;
  ::int64_t isolation_padding_bytes() const;
  void set_isolation_padding_bytes(::int64_t value);

  private:
  ::int64_t _internal_isolation_padding_bytes() const;
  void _internal_set_isolation_padding_bytes(::int64_t value);

  public:
  // uint64 isolation_order_salt = 4;
  void clear_isolation_order_salt() ;
  ::uint64_t isolation_order_salt() const;
  void set_isolation_order_salt(::uint64_t value);

  private:
  ::uint64_t _internal_isolation_order_salt() const;
  void _internal_set_isolation_order_salt(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:xla.buffer_assignment.BufferIsolationConfig)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      3, 5, 0,
      0, 2>
      _table_;


  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(
        ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena);
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena, const Impl_& from,
                          const BufferIsolationConfig& from_msg);
    ::google::protobuf::RepeatedField<::int32_t> isolation_colors_;
    mutable ::google::protobuf::internal::CachedSize _isolation_colors_cached_byte_size_;
    ::int64_t base_offset_bytes_;
    ::int64_t isolation_fuel_;
    ::int64_t isolation_padding_bytes_;
    ::uint64_t isolation_order_salt_;
    mutable ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fservice_2fbuffer_5fassignment_2eproto;
};
// -------------------------------------------------------------------

class PROTOBUF_EXPORT BufferAllocationSliceProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:xla.buffer_assignment.BufferAllocationSliceProto) */ {
 public:
  inline BufferAllocationSliceProto() : BufferAllocationSliceProto(nullptr) {}
  ~BufferAllocationSliceProto() PROTOBUF_FINAL;
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR BufferAllocationSliceProto(
      ::google::protobuf::internal::ConstantInitialized);

  inline BufferAllocationSliceProto(const BufferAllocationSliceProto& from) : BufferAllocationSliceProto(nullptr, from) {}
  inline BufferAllocationSliceProto(BufferAllocationSliceProto&& from) noexcept
      : BufferAllocationSliceProto(nullptr, std::move(from)) {}
  inline BufferAllocationSliceProto& operator=(const BufferAllocationSliceProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline BufferAllocationSliceProto& operator=(BufferAllocationSliceProto&& from) noexcept {
    if (this == &from) return *this;
    if (GetArena() == from.GetArena()
#ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetArena() != nullptr
#endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const BufferAllocationSliceProto& default_instance() {
    return *internal_default_instance();
  }
  static inline const BufferAllocationSliceProto* internal_default_instance() {
    return reinterpret_cast<const BufferAllocationSliceProto*>(
        &_BufferAllocationSliceProto_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(BufferAllocationSliceProto& a, BufferAllocationSliceProto& b) { a.Swap(&b); }
  inline void Swap(BufferAllocationSliceProto* other) {
    if (other == this) return;
#ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() != nullptr && GetArena() == other->GetArena()) {
#else   // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() == other->GetArena()) {
#endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(BufferAllocationSliceProto* other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  BufferAllocationSliceProto* New(::google::protobuf::Arena* arena = nullptr) const PROTOBUF_FINAL {
    return ::google::protobuf::Message::DefaultConstruct<BufferAllocationSliceProto>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const BufferAllocationSliceProto& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const BufferAllocationSliceProto& from) { BufferAllocationSliceProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(
      ::google::protobuf::MessageLite& to_msg,
      const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* _InternalSerialize(
      const MessageLite& msg, ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* arena);
  void SharedDtor();
  void InternalSwap(BufferAllocationSliceProto* other);
 private:
  friend class ::google::protobuf::internal::AnyMetadata;
  static ::absl::string_view FullMessageName() { return "xla.buffer_assignment.BufferAllocationSliceProto"; }

 protected:
  explicit BufferAllocationSliceProto(::google::protobuf::Arena* arena);
  BufferAllocationSliceProto(::google::protobuf::Arena* arena, const BufferAllocationSliceProto& from);
  BufferAllocationSliceProto(::google::protobuf::Arena* arena, BufferAllocationSliceProto&& from) noexcept
      : BufferAllocationSliceProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;

 public:
  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kOffsetFieldNumber = 1,
    kSizeFieldNumber = 2,
    kBufferAllocationIndexFieldNumber = 3,
  };
  // int64 offset = 1;
  void clear_offset() ;
  ::int64_t offset() const;
  void set_offset(::int64_t value);

  private:
  ::int64_t _internal_offset() const;
  void _internal_set_offset(::int64_t value);

  public:
  // int64 size = 2;
  void clear_size() ;
  ::int64_t size() const;
  void set_size(::int64_t value);

  private:
  ::int64_t _internal_size() const;
  void _internal_set_size(::int64_t value);

  public:
  // int64 buffer_allocation_index = 3;
  void clear_buffer_allocation_index() ;
  ::int64_t buffer_allocation_index() const;
  void set_buffer_allocation_index(::int64_t value);

  private:
  ::int64_t _internal_buffer_allocation_index() const;
  void _internal_set_buffer_allocation_index(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:xla.buffer_assignment.BufferAllocationSliceProto)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      2, 3, 0,
      0, 2>
      _table_;


  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(
        ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena);
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena, const Impl_& from,
                          const BufferAllocationSliceProto& from_msg);
    ::int64_t offset_;
    ::int64_t size_;
    ::int64_t buffer_allocation_index_;
    mutable ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fservice_2fbuffer_5fassignment_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// BufferIsolationConfig

// int64 base_offset_bytes = 1;
inline void BufferIsolationConfig::clear_base_offset_bytes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.base_offset_bytes_ = ::int64_t{0};
}
inline ::int64_t BufferIsolationConfig::base_offset_bytes() const {
  // @@protoc_insertion_point(field_get:xla.buffer_assignment.BufferIsolationConfig.base_offset_bytes)
  return _internal_base_offset_bytes();
}
inline void BufferIsolationConfig::set_base_offset_bytes(::int64_t value) {
  _internal_set_base_offset_bytes(value);
  // @@protoc_insertion_point(field_set:xla.buffer_assignment.BufferIsolationConfig.base_offset_bytes)
}
inline ::int64_t BufferIsolationConfig::_internal_base_offset_bytes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.base_offset_bytes_;
}
inline void BufferIsolationConfig::_internal_set_base_offset_bytes(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.base_offset_bytes_ = value;
}

// int64 isolation_fuel = 2;
inline void BufferIsolationConfig::clear_isolation_fuel() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.isolation_fuel_ = ::int64_t{0};
}
inline ::int64_t BufferIsolationConfig::isolation_fuel() const {
  // @@protoc_insertion_point(field_get:xla.buffer_assignment.BufferIsolationConfig.isolation_fuel)
  return _internal_isolation_fuel();
}
inline void BufferIsolationConfig::set_isolation_fuel(::int64_t value) {
  _internal_set_isolation_fuel(value);
  // @@protoc_insertion_point(field_set:xla.buffer_assignment.BufferIsolationConfig.isolation_fuel)
}
inline ::int64_t BufferIsolationConfig::_internal_isolation_fuel() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.isolation_fuel_;
}
inline void BufferIsolationConfig::_internal_set_isolation_fuel(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.isolation_fuel_ = value;
}

// int64 isolation_padding_bytes = 3;
inline void BufferIsolationConfig::clear_isolation_padding_bytes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.isolation_padding_bytes_ = ::int64_t{0};
}
inline ::int64_t BufferIsolationConfig::isolation_padding_bytes() const {
  // @@protoc_insertion_point(field_get:xla.buffer_assignment.BufferIsolationConfig.isolation_padding_bytes)
  return _internal_isolation_padding_bytes();
}
inline void BufferIsolationConfig::set_isolation_padding_bytes(::int64_t value) {
  _internal_set_isolation_padding_bytes(value);
  // @@protoc_insertion_point(field_set:xla.buffer_assignment.BufferIsolationConfig.isolation_padding_bytes)
}
inline ::int64_t BufferIsolationConfig::_internal_isolation_padding_bytes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.isolation_padding_bytes_;
}
inline void BufferIsolationConfig::_internal_set_isolation_padding_bytes(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.isolation_padding_bytes_ = value;
}

// uint64 isolation_order_salt = 4;
inline void BufferIsolationConfig::clear_isolation_order_salt() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.isolation_order_salt_ = ::uint64_t{0u};
}
inline ::uint64_t BufferIsolationConfig::isolation_order_salt() const {
  // @@protoc_insertion_point(field_get:xla.buffer_assignment.BufferIsolationConfig.isolation_order_salt)
  return _internal_isolation_order_salt();
}
inline void BufferIsolationConfig::set_isolation_order_salt(::uint64_t value) {
  _internal_set_isolation_order_salt(value);
  // @@protoc_insertion_point(field_set:xla.buffer_assignment.BufferIsolationConfig.isolation_order_salt)
}
inline ::uint64_t BufferIsolationConfig::_internal_isolation_order_salt() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.isolation_order_salt_;
}
inline void BufferIsolationConfig::_internal_set_isolation_order_salt(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.isolation_order_salt_ = value;
}

// repeated int32 isolation_colors = 5;
inline int BufferIsolationConfig::_internal_isolation_colors_size() const {
  return _internal_isolation_colors().size();
}
inline int BufferIsolationConfig::isolation_colors_size() const {
  return _internal_isolation_colors_size();
}
inline void BufferIsolationConfig::clear_isolation_colors() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.isolation_colors_.Clear();
}
inline ::int32_t BufferIsolationConfig::isolation_colors(int index) const {
  // @@protoc_insertion_point(field_get:xla.buffer_assignment.BufferIsolationConfig.isolation_colors)
  return _internal_isolation_colors().Get(index);
}
inline void BufferIsolationConfig::set_isolation_colors(int index, ::int32_t value) {
  _internal_mutable_isolation_colors()->Set(index, value);
  // @@protoc_insertion_point(field_set:xla.buffer_assignment.BufferIsolationConfig.isolation_colors)
}
inline void BufferIsolationConfig::add_isolation_colors(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_isolation_colors()->Add(value);
  // @@protoc_insertion_point(field_add:xla.buffer_assignment.BufferIsolationConfig.isolation_colors)
}
inline const ::google::protobuf::RepeatedField<::int32_t>& BufferIsolationConfig::isolation_colors() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:xla.buffer_assignment.BufferIsolationConfig.isolation_colors)
  return _internal_isolation_colors();
}
inline ::google::protobuf::RepeatedField<::int32_t>* BufferIsolationConfig::mutable_isolation_colors()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:xla.buffer_assignment.BufferIsolationConfig.isolation_colors)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_isolation_colors();
}
inline const ::google::protobuf::RepeatedField<::int32_t>&
BufferIsolationConfig::_internal_isolation_colors() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.isolation_colors_;
}
inline ::google::protobuf::RepeatedField<::int32_t>* BufferIsolationConfig::_internal_mutable_isolation_colors() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.isolation_colors_;
}

// -------------------------------------------------------------------

// BufferAllocationSliceProto

// int64 offset = 1;
inline void BufferAllocationSliceProto::clear_offset() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.offset_ = ::int64_t{0};
}
inline ::int64_t BufferAllocationSliceProto::offset() const {
  // @@protoc_insertion_point(field_get:xla.buffer_assignment.BufferAllocationSliceProto.offset)
  return _internal_offset();
}
inline void BufferAllocationSliceProto::set_offset(::int64_t value) {
  _internal_set_offset(value);
  // @@protoc_insertion_point(field_set:xla.buffer_assignment.BufferAllocationSliceProto.offset)
}
inline ::int64_t BufferAllocationSliceProto::_internal_offset() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.offset_;
}
inline void BufferAllocationSliceProto::_internal_set_offset(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.offset_ = value;
}

// int64 size = 2;
inline void BufferAllocationSliceProto::clear_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.size_ = ::int64_t{0};
}
inline ::int64_t BufferAllocationSliceProto::size() const {
  // @@protoc_insertion_point(field_get:xla.buffer_assignment.BufferAllocationSliceProto.size)
  return _internal_size();
}
inline void BufferAllocationSliceProto::set_size(::int64_t value) {
  _internal_set_size(value);
  // @@protoc_insertion_point(field_set:xla.buffer_assignment.BufferAllocationSliceProto.size)
}
inline ::int64_t BufferAllocationSliceProto::_internal_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.size_;
}
inline void BufferAllocationSliceProto::_internal_set_size(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.size_ = value;
}

// int64 buffer_allocation_index = 3;
inline void BufferAllocationSliceProto::clear_buffer_allocation_index() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.buffer_allocation_index_ = ::int64_t{0};
}
inline ::int64_t BufferAllocationSliceProto::buffer_allocation_index() const {
  // @@protoc_insertion_point(field_get:xla.buffer_assignment.BufferAllocationSliceProto.buffer_allocation_index)
  return _internal_buffer_allocation_index();
}
inline void BufferAllocationSliceProto::set_buffer_allocation_index(::int64_t value) {
  _internal_set_buffer_allocation_index(value);
  // @@protoc_insertion_point(field_set:xla.buffer_assignment.BufferAllocationSliceProto.buffer_allocation_index)
}
inline ::int64_t BufferAllocationSliceProto::_internal_buffer_allocation_index() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.buffer_allocation_index_;
}
inline void BufferAllocationSliceProto::_internal_set_buffer_allocation_index(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.buffer_allocation_index_ = value;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace buffer_assignment
}  // namespace xla


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // GOOGLE_PROTOBUF_INCLUDED_xla_2fservice_2fbuffer_5fassignment_2eproto_2epb_2eh
