# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.profiler namespace
"""

import sys as _sys

from tensorflow.python.profiler.model_analyzer import Profiler # line: 121
from tensorflow.python.profiler.model_analyzer import advise # line: 385
from tensorflow.python.profiler.model_analyzer import profile # line: 310
from tensorflow.python.profiler.option_builder import ProfileOptionBuilder # line: 22
from tensorflow.python.profiler.profiler import AdviceProto # line: 50
from tensorflow.python.profiler.profiler import GraphNodeProto # line: 48
from tensorflow.python.profiler.profiler import MultiGraphNodeProto # line: 49
from tensorflow.python.profiler.profiler import OpLogProto # line: 51
from tensorflow.python.profiler.tfprof_logger import write_op_log # line: 188

from tensorflow.python.util import module_wrapper as _module_wrapper

if not isinstance(_sys.modules[__name__], _module_wrapper.TFModuleWrapper):
  _sys.modules[__name__] = _module_wrapper.TFModuleWrapper(
      _sys.modules[__name__], "profiler", public_apis=None, deprecation=False,
      has_lite=False)
