# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.__operators__ namespace
"""

import sys as _sys

from tensorflow.python.ops.math_ops import _add_dispatch as add # line: 1709
from tensorflow.python.ops.math_ops import tensor_equals as eq # line: 1902
from tensorflow.python.ops.math_ops import tensor_not_equals as ne # line: 1944
from tensorflow.python.ops.ragged.ragged_getitem import ragged_tensor_getitem as ragged_getitem # line: 34
from tensorflow.python.ops.tensor_getitem_override import _slice_helper as getitem # line: 65
