// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: xla/pjrt/plugin/xla_cpu/cpu_topology.proto
// Protobuf C++ Version: 5.28.3

#ifndef GOOGLE_PROTOBUF_INCLUDED_xla_2fpjrt_2fplugin_2fxla_5fcpu_2fcpu_5ftopology_2eproto_2epb_2eh
#define GOOGLE_PROTOBUF_INCLUDED_xla_2fpjrt_2fplugin_2fxla_5fcpu_2fcpu_5ftopology_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 5028003
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_xla_2fpjrt_2fplugin_2fxla_5fcpu_2fcpu_5ftopology_2eproto PROTOBUF_EXPORT

namespace google {
namespace protobuf {
namespace internal {
class AnyMetadata;
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct PROTOBUF_EXPORT TableStruct_xla_2fpjrt_2fplugin_2fxla_5fcpu_2fcpu_5ftopology_2eproto {
  static const ::uint32_t offsets[];
};
PROTOBUF_EXPORT extern const ::google::protobuf::internal::DescriptorTable
    descriptor_table_xla_2fpjrt_2fplugin_2fxla_5fcpu_2fcpu_5ftopology_2eproto;
namespace xla {
class CpuTopologyProto;
struct CpuTopologyProtoDefaultTypeInternal;
PROTOBUF_EXPORT extern CpuTopologyProtoDefaultTypeInternal _CpuTopologyProto_default_instance_;
class CpuTopologyProto_CpuDevice;
struct CpuTopologyProto_CpuDeviceDefaultTypeInternal;
PROTOBUF_EXPORT extern CpuTopologyProto_CpuDeviceDefaultTypeInternal _CpuTopologyProto_CpuDevice_default_instance_;
}  // namespace xla
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace xla {

// ===================================================================


// -------------------------------------------------------------------

class PROTOBUF_EXPORT CpuTopologyProto_CpuDevice final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:xla.CpuTopologyProto.CpuDevice) */ {
 public:
  inline CpuTopologyProto_CpuDevice() : CpuTopologyProto_CpuDevice(nullptr) {}
  ~CpuTopologyProto_CpuDevice() PROTOBUF_FINAL;
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR CpuTopologyProto_CpuDevice(
      ::google::protobuf::internal::ConstantInitialized);

  inline CpuTopologyProto_CpuDevice(const CpuTopologyProto_CpuDevice& from) : CpuTopologyProto_CpuDevice(nullptr, from) {}
  inline CpuTopologyProto_CpuDevice(CpuTopologyProto_CpuDevice&& from) noexcept
      : CpuTopologyProto_CpuDevice(nullptr, std::move(from)) {}
  inline CpuTopologyProto_CpuDevice& operator=(const CpuTopologyProto_CpuDevice& from) {
    CopyFrom(from);
    return *this;
  }
  inline CpuTopologyProto_CpuDevice& operator=(CpuTopologyProto_CpuDevice&& from) noexcept {
    if (this == &from) return *this;
    if (GetArena() == from.GetArena()
#ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetArena() != nullptr
#endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const CpuTopologyProto_CpuDevice& default_instance() {
    return *internal_default_instance();
  }
  static inline const CpuTopologyProto_CpuDevice* internal_default_instance() {
    return reinterpret_cast<const CpuTopologyProto_CpuDevice*>(
        &_CpuTopologyProto_CpuDevice_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(CpuTopologyProto_CpuDevice& a, CpuTopologyProto_CpuDevice& b) { a.Swap(&b); }
  inline void Swap(CpuTopologyProto_CpuDevice* other) {
    if (other == this) return;
#ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() != nullptr && GetArena() == other->GetArena()) {
#else   // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() == other->GetArena()) {
#endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(CpuTopologyProto_CpuDevice* other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  CpuTopologyProto_CpuDevice* New(::google::protobuf::Arena* arena = nullptr) const PROTOBUF_FINAL {
    return ::google::protobuf::Message::DefaultConstruct<CpuTopologyProto_CpuDevice>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const CpuTopologyProto_CpuDevice& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const CpuTopologyProto_CpuDevice& from) { CpuTopologyProto_CpuDevice::MergeImpl(*this, from); }

  private:
  static void MergeImpl(
      ::google::protobuf::MessageLite& to_msg,
      const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* _InternalSerialize(
      const MessageLite& msg, ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* arena);
  void SharedDtor();
  void InternalSwap(CpuTopologyProto_CpuDevice* other);
 private:
  friend class ::google::protobuf::internal::AnyMetadata;
  static ::absl::string_view FullMessageName() { return "xla.CpuTopologyProto.CpuDevice"; }

 protected:
  explicit CpuTopologyProto_CpuDevice(::google::protobuf::Arena* arena);
  CpuTopologyProto_CpuDevice(::google::protobuf::Arena* arena, const CpuTopologyProto_CpuDevice& from);
  CpuTopologyProto_CpuDevice(::google::protobuf::Arena* arena, CpuTopologyProto_CpuDevice&& from) noexcept
      : CpuTopologyProto_CpuDevice(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;

 public:
  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kProcessIndexFieldNumber = 2,
    kLocalHardwareIdFieldNumber = 3,
  };
  // int32 process_index = 2;
  void clear_process_index() ;
  ::int32_t process_index() const;
  void set_process_index(::int32_t value);

  private:
  ::int32_t _internal_process_index() const;
  void _internal_set_process_index(::int32_t value);

  public:
  // int32 local_hardware_id = 3;
  void clear_local_hardware_id() ;
  ::int32_t local_hardware_id() const;
  void set_local_hardware_id(::int32_t value);

  private:
  ::int32_t _internal_local_hardware_id() const;
  void _internal_set_local_hardware_id(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:xla.CpuTopologyProto.CpuDevice)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      1, 2, 0,
      0, 2>
      _table_;


  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(
        ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena);
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena, const Impl_& from,
                          const CpuTopologyProto_CpuDevice& from_msg);
    ::int32_t process_index_;
    ::int32_t local_hardware_id_;
    mutable ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fpjrt_2fplugin_2fxla_5fcpu_2fcpu_5ftopology_2eproto;
};
// -------------------------------------------------------------------

class PROTOBUF_EXPORT CpuTopologyProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:xla.CpuTopologyProto) */ {
 public:
  inline CpuTopologyProto() : CpuTopologyProto(nullptr) {}
  ~CpuTopologyProto() PROTOBUF_FINAL;
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR CpuTopologyProto(
      ::google::protobuf::internal::ConstantInitialized);

  inline CpuTopologyProto(const CpuTopologyProto& from) : CpuTopologyProto(nullptr, from) {}
  inline CpuTopologyProto(CpuTopologyProto&& from) noexcept
      : CpuTopologyProto(nullptr, std::move(from)) {}
  inline CpuTopologyProto& operator=(const CpuTopologyProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline CpuTopologyProto& operator=(CpuTopologyProto&& from) noexcept {
    if (this == &from) return *this;
    if (GetArena() == from.GetArena()
#ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetArena() != nullptr
#endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const CpuTopologyProto& default_instance() {
    return *internal_default_instance();
  }
  static inline const CpuTopologyProto* internal_default_instance() {
    return reinterpret_cast<const CpuTopologyProto*>(
        &_CpuTopologyProto_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(CpuTopologyProto& a, CpuTopologyProto& b) { a.Swap(&b); }
  inline void Swap(CpuTopologyProto* other) {
    if (other == this) return;
#ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() != nullptr && GetArena() == other->GetArena()) {
#else   // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() == other->GetArena()) {
#endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(CpuTopologyProto* other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  CpuTopologyProto* New(::google::protobuf::Arena* arena = nullptr) const PROTOBUF_FINAL {
    return ::google::protobuf::Message::DefaultConstruct<CpuTopologyProto>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const CpuTopologyProto& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const CpuTopologyProto& from) { CpuTopologyProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(
      ::google::protobuf::MessageLite& to_msg,
      const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* _InternalSerialize(
      const MessageLite& msg, ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* arena);
  void SharedDtor();
  void InternalSwap(CpuTopologyProto* other);
 private:
  friend class ::google::protobuf::internal::AnyMetadata;
  static ::absl::string_view FullMessageName() { return "xla.CpuTopologyProto"; }

 protected:
  explicit CpuTopologyProto(::google::protobuf::Arena* arena);
  CpuTopologyProto(::google::protobuf::Arena* arena, const CpuTopologyProto& from);
  CpuTopologyProto(::google::protobuf::Arena* arena, CpuTopologyProto&& from) noexcept
      : CpuTopologyProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;

 public:
  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using CpuDevice = CpuTopologyProto_CpuDevice;

  // accessors -------------------------------------------------------
  enum : int {
    kCpuDevicesFieldNumber = 1,
    kMachineAttributesFieldNumber = 4,
  };
  // repeated .xla.CpuTopologyProto.CpuDevice cpu_devices = 1;
  int cpu_devices_size() const;
  private:
  int _internal_cpu_devices_size() const;

  public:
  void clear_cpu_devices() ;
  ::xla::CpuTopologyProto_CpuDevice* mutable_cpu_devices(int index);
  ::google::protobuf::RepeatedPtrField<::xla::CpuTopologyProto_CpuDevice>* mutable_cpu_devices();

  private:
  const ::google::protobuf::RepeatedPtrField<::xla::CpuTopologyProto_CpuDevice>& _internal_cpu_devices() const;
  ::google::protobuf::RepeatedPtrField<::xla::CpuTopologyProto_CpuDevice>* _internal_mutable_cpu_devices();
  public:
  const ::xla::CpuTopologyProto_CpuDevice& cpu_devices(int index) const;
  ::xla::CpuTopologyProto_CpuDevice* add_cpu_devices();
  const ::google::protobuf::RepeatedPtrField<::xla::CpuTopologyProto_CpuDevice>& cpu_devices() const;
  // repeated string machine_attributes = 4;
  int machine_attributes_size() const;
  private:
  int _internal_machine_attributes_size() const;

  public:
  void clear_machine_attributes() ;
  const std::string& machine_attributes(int index) const;
  std::string* mutable_machine_attributes(int index);
  template <typename Arg_ = const std::string&, typename... Args_>
  void set_machine_attributes(int index, Arg_&& value, Args_... args);
  std::string* add_machine_attributes();
  template <typename Arg_ = const std::string&, typename... Args_>
  void add_machine_attributes(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<std::string>& machine_attributes() const;
  ::google::protobuf::RepeatedPtrField<std::string>* mutable_machine_attributes();

  private:
  const ::google::protobuf::RepeatedPtrField<std::string>& _internal_machine_attributes() const;
  ::google::protobuf::RepeatedPtrField<std::string>* _internal_mutable_machine_attributes();

  public:
  // @@protoc_insertion_point(class_scope:xla.CpuTopologyProto)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      1, 2, 1,
      47, 2>
      _table_;


  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(
        ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena);
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena, const Impl_& from,
                          const CpuTopologyProto& from_msg);
    ::google::protobuf::RepeatedPtrField< ::xla::CpuTopologyProto_CpuDevice > cpu_devices_;
    ::google::protobuf::RepeatedPtrField<std::string> machine_attributes_;
    mutable ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fpjrt_2fplugin_2fxla_5fcpu_2fcpu_5ftopology_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// CpuTopologyProto_CpuDevice

// int32 process_index = 2;
inline void CpuTopologyProto_CpuDevice::clear_process_index() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.process_index_ = 0;
}
inline ::int32_t CpuTopologyProto_CpuDevice::process_index() const {
  // @@protoc_insertion_point(field_get:xla.CpuTopologyProto.CpuDevice.process_index)
  return _internal_process_index();
}
inline void CpuTopologyProto_CpuDevice::set_process_index(::int32_t value) {
  _internal_set_process_index(value);
  // @@protoc_insertion_point(field_set:xla.CpuTopologyProto.CpuDevice.process_index)
}
inline ::int32_t CpuTopologyProto_CpuDevice::_internal_process_index() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.process_index_;
}
inline void CpuTopologyProto_CpuDevice::_internal_set_process_index(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.process_index_ = value;
}

// int32 local_hardware_id = 3;
inline void CpuTopologyProto_CpuDevice::clear_local_hardware_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.local_hardware_id_ = 0;
}
inline ::int32_t CpuTopologyProto_CpuDevice::local_hardware_id() const {
  // @@protoc_insertion_point(field_get:xla.CpuTopologyProto.CpuDevice.local_hardware_id)
  return _internal_local_hardware_id();
}
inline void CpuTopologyProto_CpuDevice::set_local_hardware_id(::int32_t value) {
  _internal_set_local_hardware_id(value);
  // @@protoc_insertion_point(field_set:xla.CpuTopologyProto.CpuDevice.local_hardware_id)
}
inline ::int32_t CpuTopologyProto_CpuDevice::_internal_local_hardware_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.local_hardware_id_;
}
inline void CpuTopologyProto_CpuDevice::_internal_set_local_hardware_id(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.local_hardware_id_ = value;
}

// -------------------------------------------------------------------

// CpuTopologyProto

// repeated .xla.CpuTopologyProto.CpuDevice cpu_devices = 1;
inline int CpuTopologyProto::_internal_cpu_devices_size() const {
  return _internal_cpu_devices().size();
}
inline int CpuTopologyProto::cpu_devices_size() const {
  return _internal_cpu_devices_size();
}
inline void CpuTopologyProto::clear_cpu_devices() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpu_devices_.Clear();
}
inline ::xla::CpuTopologyProto_CpuDevice* CpuTopologyProto::mutable_cpu_devices(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:xla.CpuTopologyProto.cpu_devices)
  return _internal_mutable_cpu_devices()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::xla::CpuTopologyProto_CpuDevice>* CpuTopologyProto::mutable_cpu_devices()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:xla.CpuTopologyProto.cpu_devices)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_cpu_devices();
}
inline const ::xla::CpuTopologyProto_CpuDevice& CpuTopologyProto::cpu_devices(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:xla.CpuTopologyProto.cpu_devices)
  return _internal_cpu_devices().Get(index);
}
inline ::xla::CpuTopologyProto_CpuDevice* CpuTopologyProto::add_cpu_devices() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::xla::CpuTopologyProto_CpuDevice* _add = _internal_mutable_cpu_devices()->Add();
  // @@protoc_insertion_point(field_add:xla.CpuTopologyProto.cpu_devices)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::xla::CpuTopologyProto_CpuDevice>& CpuTopologyProto::cpu_devices() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:xla.CpuTopologyProto.cpu_devices)
  return _internal_cpu_devices();
}
inline const ::google::protobuf::RepeatedPtrField<::xla::CpuTopologyProto_CpuDevice>&
CpuTopologyProto::_internal_cpu_devices() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cpu_devices_;
}
inline ::google::protobuf::RepeatedPtrField<::xla::CpuTopologyProto_CpuDevice>*
CpuTopologyProto::_internal_mutable_cpu_devices() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.cpu_devices_;
}

// repeated string machine_attributes = 4;
inline int CpuTopologyProto::_internal_machine_attributes_size() const {
  return _internal_machine_attributes().size();
}
inline int CpuTopologyProto::machine_attributes_size() const {
  return _internal_machine_attributes_size();
}
inline void CpuTopologyProto::clear_machine_attributes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.machine_attributes_.Clear();
}
inline std::string* CpuTopologyProto::add_machine_attributes() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  std::string* _s = _internal_mutable_machine_attributes()->Add();
  // @@protoc_insertion_point(field_add_mutable:xla.CpuTopologyProto.machine_attributes)
  return _s;
}
inline const std::string& CpuTopologyProto::machine_attributes(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:xla.CpuTopologyProto.machine_attributes)
  return _internal_machine_attributes().Get(index);
}
inline std::string* CpuTopologyProto::mutable_machine_attributes(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:xla.CpuTopologyProto.machine_attributes)
  return _internal_mutable_machine_attributes()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void CpuTopologyProto::set_machine_attributes(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_machine_attributes()->Mutable(index),
      std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:xla.CpuTopologyProto.machine_attributes)
}
template <typename Arg_, typename... Args_>
inline void CpuTopologyProto::add_machine_attributes(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(*_internal_mutable_machine_attributes(),
                               std::forward<Arg_>(value),
                               args... );
  // @@protoc_insertion_point(field_add:xla.CpuTopologyProto.machine_attributes)
}
inline const ::google::protobuf::RepeatedPtrField<std::string>&
CpuTopologyProto::machine_attributes() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:xla.CpuTopologyProto.machine_attributes)
  return _internal_machine_attributes();
}
inline ::google::protobuf::RepeatedPtrField<std::string>*
CpuTopologyProto::mutable_machine_attributes() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:xla.CpuTopologyProto.machine_attributes)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_machine_attributes();
}
inline const ::google::protobuf::RepeatedPtrField<std::string>&
CpuTopologyProto::_internal_machine_attributes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.machine_attributes_;
}
inline ::google::protobuf::RepeatedPtrField<std::string>*
CpuTopologyProto::_internal_mutable_machine_attributes() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.machine_attributes_;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace xla


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // GOOGLE_PROTOBUF_INCLUDED_xla_2fpjrt_2fplugin_2fxla_5fcpu_2fcpu_5ftopology_2eproto_2epb_2eh
