# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.config.experimental namespace
"""

import sys as _sys

from tensorflow.python.eager.context import LogicalDeviceConfiguration as VirtualDeviceConfiguration # line: 409
from tensorflow.python.framework.config import disable_mlir_bridge # line: 926
from tensorflow.python.framework.config import enable_mlir_bridge # line: 915
from tensorflow.python.framework.config import enable_op_determinism # line: 932
from tensorflow.python.framework.config import enable_tensor_float_32_execution # line: 46
from tensorflow.python.framework.config import get_device_details # line: 757
from tensorflow.python.framework.config import get_device_policy # line: 312
from tensorflow.python.framework.config import get_memory_growth # line: 700
from tensorflow.python.framework.config import get_memory_info # line: 571
from tensorflow.python.framework.config import get_memory_usage # line: 664
from tensorflow.python.framework.config import get_synchronous_execution # line: 386
from tensorflow.python.framework.config import get_logical_device_configuration as get_virtual_device_configuration # line: 802
from tensorflow.python.framework.config import get_visible_devices # line: 500
from tensorflow.python.framework.config import list_logical_devices # line: 461
from tensorflow.python.framework.config import list_physical_devices # line: 424
from tensorflow.python.framework.config import reset_memory_stats # line: 620
from tensorflow.python.framework.config import set_device_policy # line: 340
from tensorflow.python.framework.config import set_memory_growth # line: 729
from tensorflow.python.framework.config import set_synchronous_execution # line: 399
from tensorflow.python.framework.config import set_logical_device_configuration as set_virtual_device_configuration # line: 843
from tensorflow.python.framework.config import set_visible_devices # line: 533
from tensorflow.python.framework.config import tensor_float_32_execution_enabled # line: 27
from tensorflow.python.training.server_lib import ClusterDeviceFilters # line: 495
