# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.compat namespace
"""

import sys as _sys

from tensorflow._api.v2.compat.v1.compat import v1
from tensorflow._api.v2.compat.v1.compat import v2
from tensorflow.python.compat.compat import forward_compatibility_horizon # line: 125
from tensorflow.python.compat.compat import forward_compatible # line: 65
from tensorflow.python.framework.tensor_shape import dimension_at_index # line: 139
from tensorflow.python.framework.tensor_shape import dimension_value # line: 103
from tensorflow.python.util.compat import as_bytes # line: 130
from tensorflow.python.util.compat import as_str # line: 131
from tensorflow.python.util.compat import as_str_any # line: 134
from tensorflow.python.util.compat import as_text # line: 129
from tensorflow.python.util.compat import bytes_or_text_types # line: 225
from tensorflow.python.util.compat import complex_types # line: 221
from tensorflow.python.util.compat import integral_types # line: 217
from tensorflow.python.util.compat import path_to_str # line: 154
from tensorflow.python.util.compat import real_types # line: 219

from tensorflow.python.util import module_wrapper as _module_wrapper

if not isinstance(_sys.modules[__name__], _module_wrapper.TFModuleWrapper):
  _sys.modules[__name__] = _module_wrapper.TFModuleWrapper(
      _sys.modules[__name__], "compat", public_apis=None, deprecation=False,
      has_lite=False)
