# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.distribute.experimental.partitioners namespace
"""

import sys as _sys

from tensorflow.python.distribute.sharded_variable import FixedShardsPartitioner # line: 83
from tensorflow.python.distribute.sharded_variable import MaxSizePartitioner # line: 175
from tensorflow.python.distribute.sharded_variable import MinSizePartitioner # line: 114
from tensorflow.python.distribute.sharded_variable import Partitioner # line: 46
