# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.nest namespace
"""

import sys as _sys

from tensorflow.python.util.nest import assert_same_structure # line: 298
from tensorflow.python.util.nest import flatten # line: 198
from tensorflow.python.util.nest import is_nested # line: 163
from tensorflow.python.util.nest import map_structure # line: 542
from tensorflow.python.util.nest import pack_sequence_as # line: 423
