# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.metrics namespace
"""

import sys as _sys

from tensorflow.python.ops.metrics_impl import accuracy # line: 481
from tensorflow.python.ops.metrics_impl import auc # line: 805
from tensorflow.python.ops.metrics_impl import average_precision_at_k # line: 3429
from tensorflow.python.ops.metrics_impl import false_negatives # line: 1701
from tensorflow.python.ops.metrics_impl import false_negatives_at_thresholds # line: 1753
from tensorflow.python.ops.metrics_impl import false_positives # line: 1809
from tensorflow.python.ops.metrics_impl import false_positives_at_thresholds # line: 1862
from tensorflow.python.ops.metrics_impl import mean # line: 315
from tensorflow.python.ops.metrics_impl import mean_absolute_error # line: 1042
from tensorflow.python.ops.metrics_impl import mean_cosine_distance # line: 1103
from tensorflow.python.ops.metrics_impl import mean_iou # line: 1282
from tensorflow.python.ops.metrics_impl import mean_per_class_accuracy # line: 1177
from tensorflow.python.ops.metrics_impl import mean_relative_error # line: 1385
from tensorflow.python.ops.metrics_impl import mean_squared_error # line: 1454
from tensorflow.python.ops.metrics_impl import mean_tensor # line: 1515
from tensorflow.python.ops.metrics_impl import percentage_below # line: 1600
from tensorflow.python.ops.metrics_impl import precision # line: 2136
from tensorflow.python.ops.metrics_impl import precision_at_k # line: 3720
from tensorflow.python.ops.metrics_impl import precision_at_thresholds # line: 2231
from tensorflow.python.ops.metrics_impl import precision_at_top_k # line: 3609
from tensorflow.python.ops.metrics_impl import recall # line: 2312
from tensorflow.python.ops.metrics_impl import recall_at_k # line: 2663
from tensorflow.python.ops.metrics_impl import recall_at_thresholds # line: 2840
from tensorflow.python.ops.metrics_impl import recall_at_top_k # line: 2756
from tensorflow.python.ops.metrics_impl import root_mean_squared_error # line: 2918
from tensorflow.python.ops.metrics_impl import sensitivity_at_specificity # line: 2989
from tensorflow.python.ops.metrics_impl import sparse_average_precision_at_k # line: 3409
from tensorflow.python.ops.metrics_impl import sparse_precision_at_k # line: 3698
from tensorflow.python.ops.metrics_impl import specificity_at_sensitivity # line: 3814
from tensorflow.python.ops.metrics_impl import true_negatives # line: 1918
from tensorflow.python.ops.metrics_impl import true_negatives_at_thresholds # line: 1971
from tensorflow.python.ops.metrics_impl import true_positives # line: 2027
from tensorflow.python.ops.metrics_impl import true_positives_at_thresholds # line: 2080

from tensorflow.python.util import module_wrapper as _module_wrapper

if not isinstance(_sys.modules[__name__], _module_wrapper.TFModuleWrapper):
  _sys.modules[__name__] = _module_wrapper.TFModuleWrapper(
      _sys.modules[__name__], "metrics", public_apis=None, deprecation=False,
      has_lite=False)
