# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.queue namespace
"""

import sys as _sys

from tensorflow.python.ops.data_flow_ops import FIFOQueue # line: 772
from tensorflow.python.ops.data_flow_ops import PaddingFIFOQueue # line: 912
from tensorflow.python.ops.data_flow_ops import PriorityQueue # line: 988
from tensorflow.python.ops.data_flow_ops import QueueBase # line: 115
from tensorflow.python.ops.data_flow_ops import RandomShuffleQueue # line: 682
