// Generated by the gRPC C++ plugin.
// If you make any local change, they will be lost.
// source: tensorflow/core/protobuf/eager_service.proto

#include "tensorflow/core/protobuf/eager_service.pb.h"
#include "tensorflow/core/protobuf/eager_service.grpc.pb.h"

#include <functional>
#include <grpcpp/support/async_stream.h>
#include <grpcpp/support/async_unary_call.h>
#include <grpcpp/impl/channel_interface.h>
#include <grpcpp/impl/client_unary_call.h>
#include <grpcpp/support/client_callback.h>
#include <grpcpp/support/message_allocator.h>
#include <grpcpp/support/method_handler.h>
#include <grpcpp/impl/rpc_service_method.h>
#include <grpcpp/support/server_callback.h>
#include <grpcpp/impl/server_callback_handlers.h>
#include <grpcpp/server_context.h>
#include <grpcpp/impl/service_type.h>
#include <grpcpp/support/sync_stream.h>
namespace tensorflow {
namespace eager {

static const char* grpcEagerService_method_names[] = {
  "/tensorflow.eager.EagerService/CreateContext",
  "/tensorflow.eager.EagerService/UpdateContext",
  "/tensorflow.eager.EagerService/Enqueue",
  "/tensorflow.eager.EagerService/StreamingEnqueue",
  "/tensorflow.eager.EagerService/WaitQueueDone",
  "/tensorflow.eager.EagerService/RunComponentFunction",
  "/tensorflow.eager.EagerService/KeepAlive",
  "/tensorflow.eager.EagerService/CloseContext",
};

std::unique_ptr< grpc::EagerService::Stub> grpc::EagerService::NewStub(const std::shared_ptr< ::grpc::ChannelInterface>& channel, const ::grpc::StubOptions& options) {
  (void)options;
  std::unique_ptr< grpc::EagerService::Stub> stub(new grpc::EagerService::Stub(channel, options));
  return stub;
}

grpc::EagerService::Stub::Stub(const std::shared_ptr< ::grpc::ChannelInterface>& channel, const ::grpc::StubOptions& options)
  : channel_(channel), rpcmethod_CreateContext_(grpcEagerService_method_names[0], options.suffix_for_stats(),::grpc::internal::RpcMethod::NORMAL_RPC, channel)
  , rpcmethod_UpdateContext_(grpcEagerService_method_names[1], options.suffix_for_stats(),::grpc::internal::RpcMethod::NORMAL_RPC, channel)
  , rpcmethod_Enqueue_(grpcEagerService_method_names[2], options.suffix_for_stats(),::grpc::internal::RpcMethod::NORMAL_RPC, channel)
  , rpcmethod_StreamingEnqueue_(grpcEagerService_method_names[3], options.suffix_for_stats(),::grpc::internal::RpcMethod::BIDI_STREAMING, channel)
  , rpcmethod_WaitQueueDone_(grpcEagerService_method_names[4], options.suffix_for_stats(),::grpc::internal::RpcMethod::NORMAL_RPC, channel)
  , rpcmethod_RunComponentFunction_(grpcEagerService_method_names[5], options.suffix_for_stats(),::grpc::internal::RpcMethod::NORMAL_RPC, channel)
  , rpcmethod_KeepAlive_(grpcEagerService_method_names[6], options.suffix_for_stats(),::grpc::internal::RpcMethod::NORMAL_RPC, channel)
  , rpcmethod_CloseContext_(grpcEagerService_method_names[7], options.suffix_for_stats(),::grpc::internal::RpcMethod::NORMAL_RPC, channel)
  {}

::grpc::Status grpc::EagerService::Stub::CreateContext(::grpc::ClientContext* context, const ::tensorflow::eager::CreateContextRequest& request, ::tensorflow::eager::CreateContextResponse* response) {
  return ::grpc::internal::BlockingUnaryCall< ::tensorflow::eager::CreateContextRequest, ::tensorflow::eager::CreateContextResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), rpcmethod_CreateContext_, context, request, response);
}

void grpc::EagerService::Stub::async::CreateContext(::grpc::ClientContext* context, const ::tensorflow::eager::CreateContextRequest* request, ::tensorflow::eager::CreateContextResponse* response, std::function<void(::grpc::Status)> f) {
  ::grpc::internal::CallbackUnaryCall< ::tensorflow::eager::CreateContextRequest, ::tensorflow::eager::CreateContextResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_CreateContext_, context, request, response, std::move(f));
}

void grpc::EagerService::Stub::async::CreateContext(::grpc::ClientContext* context, const ::tensorflow::eager::CreateContextRequest* request, ::tensorflow::eager::CreateContextResponse* response, ::grpc::ClientUnaryReactor* reactor) {
  ::grpc::internal::ClientCallbackUnaryFactory::Create< ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_CreateContext_, context, request, response, reactor);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::CreateContextResponse>* grpc::EagerService::Stub::PrepareAsyncCreateContextRaw(::grpc::ClientContext* context, const ::tensorflow::eager::CreateContextRequest& request, ::grpc::CompletionQueue* cq) {
  return ::grpc::internal::ClientAsyncResponseReaderHelper::Create< ::tensorflow::eager::CreateContextResponse, ::tensorflow::eager::CreateContextRequest, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), cq, rpcmethod_CreateContext_, context, request);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::CreateContextResponse>* grpc::EagerService::Stub::AsyncCreateContextRaw(::grpc::ClientContext* context, const ::tensorflow::eager::CreateContextRequest& request, ::grpc::CompletionQueue* cq) {
  auto* result =
    this->PrepareAsyncCreateContextRaw(context, request, cq);
  result->StartCall();
  return result;
}

::grpc::Status grpc::EagerService::Stub::UpdateContext(::grpc::ClientContext* context, const ::tensorflow::eager::UpdateContextRequest& request, ::tensorflow::eager::UpdateContextResponse* response) {
  return ::grpc::internal::BlockingUnaryCall< ::tensorflow::eager::UpdateContextRequest, ::tensorflow::eager::UpdateContextResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), rpcmethod_UpdateContext_, context, request, response);
}

void grpc::EagerService::Stub::async::UpdateContext(::grpc::ClientContext* context, const ::tensorflow::eager::UpdateContextRequest* request, ::tensorflow::eager::UpdateContextResponse* response, std::function<void(::grpc::Status)> f) {
  ::grpc::internal::CallbackUnaryCall< ::tensorflow::eager::UpdateContextRequest, ::tensorflow::eager::UpdateContextResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_UpdateContext_, context, request, response, std::move(f));
}

void grpc::EagerService::Stub::async::UpdateContext(::grpc::ClientContext* context, const ::tensorflow::eager::UpdateContextRequest* request, ::tensorflow::eager::UpdateContextResponse* response, ::grpc::ClientUnaryReactor* reactor) {
  ::grpc::internal::ClientCallbackUnaryFactory::Create< ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_UpdateContext_, context, request, response, reactor);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::UpdateContextResponse>* grpc::EagerService::Stub::PrepareAsyncUpdateContextRaw(::grpc::ClientContext* context, const ::tensorflow::eager::UpdateContextRequest& request, ::grpc::CompletionQueue* cq) {
  return ::grpc::internal::ClientAsyncResponseReaderHelper::Create< ::tensorflow::eager::UpdateContextResponse, ::tensorflow::eager::UpdateContextRequest, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), cq, rpcmethod_UpdateContext_, context, request);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::UpdateContextResponse>* grpc::EagerService::Stub::AsyncUpdateContextRaw(::grpc::ClientContext* context, const ::tensorflow::eager::UpdateContextRequest& request, ::grpc::CompletionQueue* cq) {
  auto* result =
    this->PrepareAsyncUpdateContextRaw(context, request, cq);
  result->StartCall();
  return result;
}

::grpc::Status grpc::EagerService::Stub::Enqueue(::grpc::ClientContext* context, const ::tensorflow::eager::EnqueueRequest& request, ::tensorflow::eager::EnqueueResponse* response) {
  return ::grpc::internal::BlockingUnaryCall< ::tensorflow::eager::EnqueueRequest, ::tensorflow::eager::EnqueueResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), rpcmethod_Enqueue_, context, request, response);
}

void grpc::EagerService::Stub::async::Enqueue(::grpc::ClientContext* context, const ::tensorflow::eager::EnqueueRequest* request, ::tensorflow::eager::EnqueueResponse* response, std::function<void(::grpc::Status)> f) {
  ::grpc::internal::CallbackUnaryCall< ::tensorflow::eager::EnqueueRequest, ::tensorflow::eager::EnqueueResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_Enqueue_, context, request, response, std::move(f));
}

void grpc::EagerService::Stub::async::Enqueue(::grpc::ClientContext* context, const ::tensorflow::eager::EnqueueRequest* request, ::tensorflow::eager::EnqueueResponse* response, ::grpc::ClientUnaryReactor* reactor) {
  ::grpc::internal::ClientCallbackUnaryFactory::Create< ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_Enqueue_, context, request, response, reactor);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::EnqueueResponse>* grpc::EagerService::Stub::PrepareAsyncEnqueueRaw(::grpc::ClientContext* context, const ::tensorflow::eager::EnqueueRequest& request, ::grpc::CompletionQueue* cq) {
  return ::grpc::internal::ClientAsyncResponseReaderHelper::Create< ::tensorflow::eager::EnqueueResponse, ::tensorflow::eager::EnqueueRequest, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), cq, rpcmethod_Enqueue_, context, request);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::EnqueueResponse>* grpc::EagerService::Stub::AsyncEnqueueRaw(::grpc::ClientContext* context, const ::tensorflow::eager::EnqueueRequest& request, ::grpc::CompletionQueue* cq) {
  auto* result =
    this->PrepareAsyncEnqueueRaw(context, request, cq);
  result->StartCall();
  return result;
}

::grpc::ClientReaderWriter< ::tensorflow::eager::EnqueueRequest, ::tensorflow::eager::EnqueueResponse>* grpc::EagerService::Stub::StreamingEnqueueRaw(::grpc::ClientContext* context) {
  return ::grpc::internal::ClientReaderWriterFactory< ::tensorflow::eager::EnqueueRequest, ::tensorflow::eager::EnqueueResponse>::Create(channel_.get(), rpcmethod_StreamingEnqueue_, context);
}

void grpc::EagerService::Stub::async::StreamingEnqueue(::grpc::ClientContext* context, ::grpc::ClientBidiReactor< ::tensorflow::eager::EnqueueRequest,::tensorflow::eager::EnqueueResponse>* reactor) {
  ::grpc::internal::ClientCallbackReaderWriterFactory< ::tensorflow::eager::EnqueueRequest,::tensorflow::eager::EnqueueResponse>::Create(stub_->channel_.get(), stub_->rpcmethod_StreamingEnqueue_, context, reactor);
}

::grpc::ClientAsyncReaderWriter< ::tensorflow::eager::EnqueueRequest, ::tensorflow::eager::EnqueueResponse>* grpc::EagerService::Stub::AsyncStreamingEnqueueRaw(::grpc::ClientContext* context, ::grpc::CompletionQueue* cq, void* tag) {
  return ::grpc::internal::ClientAsyncReaderWriterFactory< ::tensorflow::eager::EnqueueRequest, ::tensorflow::eager::EnqueueResponse>::Create(channel_.get(), cq, rpcmethod_StreamingEnqueue_, context, true, tag);
}

::grpc::ClientAsyncReaderWriter< ::tensorflow::eager::EnqueueRequest, ::tensorflow::eager::EnqueueResponse>* grpc::EagerService::Stub::PrepareAsyncStreamingEnqueueRaw(::grpc::ClientContext* context, ::grpc::CompletionQueue* cq) {
  return ::grpc::internal::ClientAsyncReaderWriterFactory< ::tensorflow::eager::EnqueueRequest, ::tensorflow::eager::EnqueueResponse>::Create(channel_.get(), cq, rpcmethod_StreamingEnqueue_, context, false, nullptr);
}

::grpc::Status grpc::EagerService::Stub::WaitQueueDone(::grpc::ClientContext* context, const ::tensorflow::eager::WaitQueueDoneRequest& request, ::tensorflow::eager::WaitQueueDoneResponse* response) {
  return ::grpc::internal::BlockingUnaryCall< ::tensorflow::eager::WaitQueueDoneRequest, ::tensorflow::eager::WaitQueueDoneResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), rpcmethod_WaitQueueDone_, context, request, response);
}

void grpc::EagerService::Stub::async::WaitQueueDone(::grpc::ClientContext* context, const ::tensorflow::eager::WaitQueueDoneRequest* request, ::tensorflow::eager::WaitQueueDoneResponse* response, std::function<void(::grpc::Status)> f) {
  ::grpc::internal::CallbackUnaryCall< ::tensorflow::eager::WaitQueueDoneRequest, ::tensorflow::eager::WaitQueueDoneResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_WaitQueueDone_, context, request, response, std::move(f));
}

void grpc::EagerService::Stub::async::WaitQueueDone(::grpc::ClientContext* context, const ::tensorflow::eager::WaitQueueDoneRequest* request, ::tensorflow::eager::WaitQueueDoneResponse* response, ::grpc::ClientUnaryReactor* reactor) {
  ::grpc::internal::ClientCallbackUnaryFactory::Create< ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_WaitQueueDone_, context, request, response, reactor);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::WaitQueueDoneResponse>* grpc::EagerService::Stub::PrepareAsyncWaitQueueDoneRaw(::grpc::ClientContext* context, const ::tensorflow::eager::WaitQueueDoneRequest& request, ::grpc::CompletionQueue* cq) {
  return ::grpc::internal::ClientAsyncResponseReaderHelper::Create< ::tensorflow::eager::WaitQueueDoneResponse, ::tensorflow::eager::WaitQueueDoneRequest, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), cq, rpcmethod_WaitQueueDone_, context, request);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::WaitQueueDoneResponse>* grpc::EagerService::Stub::AsyncWaitQueueDoneRaw(::grpc::ClientContext* context, const ::tensorflow::eager::WaitQueueDoneRequest& request, ::grpc::CompletionQueue* cq) {
  auto* result =
    this->PrepareAsyncWaitQueueDoneRaw(context, request, cq);
  result->StartCall();
  return result;
}

::grpc::Status grpc::EagerService::Stub::RunComponentFunction(::grpc::ClientContext* context, const ::tensorflow::eager::RunComponentFunctionRequest& request, ::tensorflow::eager::RunComponentFunctionResponse* response) {
  return ::grpc::internal::BlockingUnaryCall< ::tensorflow::eager::RunComponentFunctionRequest, ::tensorflow::eager::RunComponentFunctionResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), rpcmethod_RunComponentFunction_, context, request, response);
}

void grpc::EagerService::Stub::async::RunComponentFunction(::grpc::ClientContext* context, const ::tensorflow::eager::RunComponentFunctionRequest* request, ::tensorflow::eager::RunComponentFunctionResponse* response, std::function<void(::grpc::Status)> f) {
  ::grpc::internal::CallbackUnaryCall< ::tensorflow::eager::RunComponentFunctionRequest, ::tensorflow::eager::RunComponentFunctionResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_RunComponentFunction_, context, request, response, std::move(f));
}

void grpc::EagerService::Stub::async::RunComponentFunction(::grpc::ClientContext* context, const ::tensorflow::eager::RunComponentFunctionRequest* request, ::tensorflow::eager::RunComponentFunctionResponse* response, ::grpc::ClientUnaryReactor* reactor) {
  ::grpc::internal::ClientCallbackUnaryFactory::Create< ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_RunComponentFunction_, context, request, response, reactor);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::RunComponentFunctionResponse>* grpc::EagerService::Stub::PrepareAsyncRunComponentFunctionRaw(::grpc::ClientContext* context, const ::tensorflow::eager::RunComponentFunctionRequest& request, ::grpc::CompletionQueue* cq) {
  return ::grpc::internal::ClientAsyncResponseReaderHelper::Create< ::tensorflow::eager::RunComponentFunctionResponse, ::tensorflow::eager::RunComponentFunctionRequest, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), cq, rpcmethod_RunComponentFunction_, context, request);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::RunComponentFunctionResponse>* grpc::EagerService::Stub::AsyncRunComponentFunctionRaw(::grpc::ClientContext* context, const ::tensorflow::eager::RunComponentFunctionRequest& request, ::grpc::CompletionQueue* cq) {
  auto* result =
    this->PrepareAsyncRunComponentFunctionRaw(context, request, cq);
  result->StartCall();
  return result;
}

::grpc::Status grpc::EagerService::Stub::KeepAlive(::grpc::ClientContext* context, const ::tensorflow::eager::KeepAliveRequest& request, ::tensorflow::eager::KeepAliveResponse* response) {
  return ::grpc::internal::BlockingUnaryCall< ::tensorflow::eager::KeepAliveRequest, ::tensorflow::eager::KeepAliveResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), rpcmethod_KeepAlive_, context, request, response);
}

void grpc::EagerService::Stub::async::KeepAlive(::grpc::ClientContext* context, const ::tensorflow::eager::KeepAliveRequest* request, ::tensorflow::eager::KeepAliveResponse* response, std::function<void(::grpc::Status)> f) {
  ::grpc::internal::CallbackUnaryCall< ::tensorflow::eager::KeepAliveRequest, ::tensorflow::eager::KeepAliveResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_KeepAlive_, context, request, response, std::move(f));
}

void grpc::EagerService::Stub::async::KeepAlive(::grpc::ClientContext* context, const ::tensorflow::eager::KeepAliveRequest* request, ::tensorflow::eager::KeepAliveResponse* response, ::grpc::ClientUnaryReactor* reactor) {
  ::grpc::internal::ClientCallbackUnaryFactory::Create< ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_KeepAlive_, context, request, response, reactor);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::KeepAliveResponse>* grpc::EagerService::Stub::PrepareAsyncKeepAliveRaw(::grpc::ClientContext* context, const ::tensorflow::eager::KeepAliveRequest& request, ::grpc::CompletionQueue* cq) {
  return ::grpc::internal::ClientAsyncResponseReaderHelper::Create< ::tensorflow::eager::KeepAliveResponse, ::tensorflow::eager::KeepAliveRequest, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), cq, rpcmethod_KeepAlive_, context, request);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::KeepAliveResponse>* grpc::EagerService::Stub::AsyncKeepAliveRaw(::grpc::ClientContext* context, const ::tensorflow::eager::KeepAliveRequest& request, ::grpc::CompletionQueue* cq) {
  auto* result =
    this->PrepareAsyncKeepAliveRaw(context, request, cq);
  result->StartCall();
  return result;
}

::grpc::Status grpc::EagerService::Stub::CloseContext(::grpc::ClientContext* context, const ::tensorflow::eager::CloseContextRequest& request, ::tensorflow::eager::CloseContextResponse* response) {
  return ::grpc::internal::BlockingUnaryCall< ::tensorflow::eager::CloseContextRequest, ::tensorflow::eager::CloseContextResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), rpcmethod_CloseContext_, context, request, response);
}

void grpc::EagerService::Stub::async::CloseContext(::grpc::ClientContext* context, const ::tensorflow::eager::CloseContextRequest* request, ::tensorflow::eager::CloseContextResponse* response, std::function<void(::grpc::Status)> f) {
  ::grpc::internal::CallbackUnaryCall< ::tensorflow::eager::CloseContextRequest, ::tensorflow::eager::CloseContextResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_CloseContext_, context, request, response, std::move(f));
}

void grpc::EagerService::Stub::async::CloseContext(::grpc::ClientContext* context, const ::tensorflow::eager::CloseContextRequest* request, ::tensorflow::eager::CloseContextResponse* response, ::grpc::ClientUnaryReactor* reactor) {
  ::grpc::internal::ClientCallbackUnaryFactory::Create< ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(stub_->channel_.get(), stub_->rpcmethod_CloseContext_, context, request, response, reactor);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::CloseContextResponse>* grpc::EagerService::Stub::PrepareAsyncCloseContextRaw(::grpc::ClientContext* context, const ::tensorflow::eager::CloseContextRequest& request, ::grpc::CompletionQueue* cq) {
  return ::grpc::internal::ClientAsyncResponseReaderHelper::Create< ::tensorflow::eager::CloseContextResponse, ::tensorflow::eager::CloseContextRequest, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(channel_.get(), cq, rpcmethod_CloseContext_, context, request);
}

::grpc::ClientAsyncResponseReader< ::tensorflow::eager::CloseContextResponse>* grpc::EagerService::Stub::AsyncCloseContextRaw(::grpc::ClientContext* context, const ::tensorflow::eager::CloseContextRequest& request, ::grpc::CompletionQueue* cq) {
  auto* result =
    this->PrepareAsyncCloseContextRaw(context, request, cq);
  result->StartCall();
  return result;
}

grpc::EagerService::Service::Service() {
  AddMethod(new ::grpc::internal::RpcServiceMethod(
      grpcEagerService_method_names[0],
      ::grpc::internal::RpcMethod::NORMAL_RPC,
      new ::grpc::internal::RpcMethodHandler< grpc::EagerService::Service, ::tensorflow::eager::CreateContextRequest, ::tensorflow::eager::CreateContextResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(
          [](grpc::EagerService::Service* service,
             ::grpc::ServerContext* ctx,
             const ::tensorflow::eager::CreateContextRequest* req,
             ::tensorflow::eager::CreateContextResponse* resp) {
               return service->CreateContext(ctx, req, resp);
             }, this)));
  AddMethod(new ::grpc::internal::RpcServiceMethod(
      grpcEagerService_method_names[1],
      ::grpc::internal::RpcMethod::NORMAL_RPC,
      new ::grpc::internal::RpcMethodHandler< grpc::EagerService::Service, ::tensorflow::eager::UpdateContextRequest, ::tensorflow::eager::UpdateContextResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(
          [](grpc::EagerService::Service* service,
             ::grpc::ServerContext* ctx,
             const ::tensorflow::eager::UpdateContextRequest* req,
             ::tensorflow::eager::UpdateContextResponse* resp) {
               return service->UpdateContext(ctx, req, resp);
             }, this)));
  AddMethod(new ::grpc::internal::RpcServiceMethod(
      grpcEagerService_method_names[2],
      ::grpc::internal::RpcMethod::NORMAL_RPC,
      new ::grpc::internal::RpcMethodHandler< grpc::EagerService::Service, ::tensorflow::eager::EnqueueRequest, ::tensorflow::eager::EnqueueResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(
          [](grpc::EagerService::Service* service,
             ::grpc::ServerContext* ctx,
             const ::tensorflow::eager::EnqueueRequest* req,
             ::tensorflow::eager::EnqueueResponse* resp) {
               return service->Enqueue(ctx, req, resp);
             }, this)));
  AddMethod(new ::grpc::internal::RpcServiceMethod(
      grpcEagerService_method_names[3],
      ::grpc::internal::RpcMethod::BIDI_STREAMING,
      new ::grpc::internal::BidiStreamingHandler< grpc::EagerService::Service, ::tensorflow::eager::EnqueueRequest, ::tensorflow::eager::EnqueueResponse>(
          [](grpc::EagerService::Service* service,
             ::grpc::ServerContext* ctx,
             ::grpc::ServerReaderWriter<::tensorflow::eager::EnqueueResponse,
             ::tensorflow::eager::EnqueueRequest>* stream) {
               return service->StreamingEnqueue(ctx, stream);
             }, this)));
  AddMethod(new ::grpc::internal::RpcServiceMethod(
      grpcEagerService_method_names[4],
      ::grpc::internal::RpcMethod::NORMAL_RPC,
      new ::grpc::internal::RpcMethodHandler< grpc::EagerService::Service, ::tensorflow::eager::WaitQueueDoneRequest, ::tensorflow::eager::WaitQueueDoneResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(
          [](grpc::EagerService::Service* service,
             ::grpc::ServerContext* ctx,
             const ::tensorflow::eager::WaitQueueDoneRequest* req,
             ::tensorflow::eager::WaitQueueDoneResponse* resp) {
               return service->WaitQueueDone(ctx, req, resp);
             }, this)));
  AddMethod(new ::grpc::internal::RpcServiceMethod(
      grpcEagerService_method_names[5],
      ::grpc::internal::RpcMethod::NORMAL_RPC,
      new ::grpc::internal::RpcMethodHandler< grpc::EagerService::Service, ::tensorflow::eager::RunComponentFunctionRequest, ::tensorflow::eager::RunComponentFunctionResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(
          [](grpc::EagerService::Service* service,
             ::grpc::ServerContext* ctx,
             const ::tensorflow::eager::RunComponentFunctionRequest* req,
             ::tensorflow::eager::RunComponentFunctionResponse* resp) {
               return service->RunComponentFunction(ctx, req, resp);
             }, this)));
  AddMethod(new ::grpc::internal::RpcServiceMethod(
      grpcEagerService_method_names[6],
      ::grpc::internal::RpcMethod::NORMAL_RPC,
      new ::grpc::internal::RpcMethodHandler< grpc::EagerService::Service, ::tensorflow::eager::KeepAliveRequest, ::tensorflow::eager::KeepAliveResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(
          [](grpc::EagerService::Service* service,
             ::grpc::ServerContext* ctx,
             const ::tensorflow::eager::KeepAliveRequest* req,
             ::tensorflow::eager::KeepAliveResponse* resp) {
               return service->KeepAlive(ctx, req, resp);
             }, this)));
  AddMethod(new ::grpc::internal::RpcServiceMethod(
      grpcEagerService_method_names[7],
      ::grpc::internal::RpcMethod::NORMAL_RPC,
      new ::grpc::internal::RpcMethodHandler< grpc::EagerService::Service, ::tensorflow::eager::CloseContextRequest, ::tensorflow::eager::CloseContextResponse, ::grpc::protobuf::MessageLite, ::grpc::protobuf::MessageLite>(
          [](grpc::EagerService::Service* service,
             ::grpc::ServerContext* ctx,
             const ::tensorflow::eager::CloseContextRequest* req,
             ::tensorflow::eager::CloseContextResponse* resp) {
               return service->CloseContext(ctx, req, resp);
             }, this)));
}

grpc::EagerService::Service::~Service() {
}

::grpc::Status grpc::EagerService::Service::CreateContext(::grpc::ServerContext* context, const ::tensorflow::eager::CreateContextRequest* request, ::tensorflow::eager::CreateContextResponse* response) {
  (void) context;
  (void) request;
  (void) response;
  return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
}

::grpc::Status grpc::EagerService::Service::UpdateContext(::grpc::ServerContext* context, const ::tensorflow::eager::UpdateContextRequest* request, ::tensorflow::eager::UpdateContextResponse* response) {
  (void) context;
  (void) request;
  (void) response;
  return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
}

::grpc::Status grpc::EagerService::Service::Enqueue(::grpc::ServerContext* context, const ::tensorflow::eager::EnqueueRequest* request, ::tensorflow::eager::EnqueueResponse* response) {
  (void) context;
  (void) request;
  (void) response;
  return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
}

::grpc::Status grpc::EagerService::Service::StreamingEnqueue(::grpc::ServerContext* context, ::grpc::ServerReaderWriter< ::tensorflow::eager::EnqueueResponse, ::tensorflow::eager::EnqueueRequest>* stream) {
  (void) context;
  (void) stream;
  return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
}

::grpc::Status grpc::EagerService::Service::WaitQueueDone(::grpc::ServerContext* context, const ::tensorflow::eager::WaitQueueDoneRequest* request, ::tensorflow::eager::WaitQueueDoneResponse* response) {
  (void) context;
  (void) request;
  (void) response;
  return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
}

::grpc::Status grpc::EagerService::Service::RunComponentFunction(::grpc::ServerContext* context, const ::tensorflow::eager::RunComponentFunctionRequest* request, ::tensorflow::eager::RunComponentFunctionResponse* response) {
  (void) context;
  (void) request;
  (void) response;
  return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
}

::grpc::Status grpc::EagerService::Service::KeepAlive(::grpc::ServerContext* context, const ::tensorflow::eager::KeepAliveRequest* request, ::tensorflow::eager::KeepAliveResponse* response) {
  (void) context;
  (void) request;
  (void) response;
  return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
}

::grpc::Status grpc::EagerService::Service::CloseContext(::grpc::ServerContext* context, const ::tensorflow::eager::CloseContextRequest* request, ::tensorflow::eager::CloseContextResponse* response) {
  (void) context;
  (void) request;
  (void) response;
  return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
}


}  // namespace tensorflow
}  // namespace eager

