# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tensorflow/core/util/quantization/uniform_quant_ops_attr.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'tensorflow/core/util/quantization/uniform_quant_ops_attr.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n>tensorflow/core/util/quantization/uniform_quant_ops_attr.proto\x12\ntensorflow\"\xec\x02\n/UniformQuantizedConvolutionDimensionNumbersAttr\x12\x1d\n\x15input_batch_dimension\x18\x01 \x01(\x03\x12\x1f\n\x17input_feature_dimension\x18\x02 \x01(\x03\x12 \n\x18input_spatial_dimensions\x18\x03 \x03(\x03\x12&\n\x1ekernel_input_feature_dimension\x18\x04 \x01(\x03\x12\'\n\x1fkernel_output_feature_dimension\x18\x05 \x01(\x03\x12!\n\x19kernel_spatial_dimensions\x18\x06 \x03(\x03\x12\x1e\n\x16output_batch_dimension\x18\x07 \x01(\x03\x12 \n\x18output_feature_dimension\x18\x08 \x01(\x03\x12!\n\x19output_spatial_dimensions\x18\t \x03(\x03\x42WZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.util.quantization.uniform_quant_ops_attr_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto'
  _globals['_UNIFORMQUANTIZEDCONVOLUTIONDIMENSIONNUMBERSATTR']._serialized_start=79
  _globals['_UNIFORMQUANTIZEDCONVOLUTIONDIMENSIONNUMBERSATTR']._serialized_end=443
# @@protoc_insertion_point(module_scope)
