// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: xla/pjrt/proto/pjrt_value_type.proto
// Protobuf C++ Version: 5.28.3

#ifndef GOOGLE_PROTOBUF_INCLUDED_xla_2fpjrt_2fproto_2fpjrt_5fvalue_5ftype_2eproto_2epb_2eh
#define GOOGLE_PROTOBUF_INCLUDED_xla_2fpjrt_2fproto_2fpjrt_5fvalue_5ftype_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 5028003
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_xla_2fpjrt_2fproto_2fpjrt_5fvalue_5ftype_2eproto PROTOBUF_EXPORT

namespace google {
namespace protobuf {
namespace internal {
class AnyMetadata;
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct PROTOBUF_EXPORT TableStruct_xla_2fpjrt_2fproto_2fpjrt_5fvalue_5ftype_2eproto {
  static const ::uint32_t offsets[];
};
PROTOBUF_EXPORT extern const ::google::protobuf::internal::DescriptorTable
    descriptor_table_xla_2fpjrt_2fproto_2fpjrt_5fvalue_5ftype_2eproto;
namespace xla {
class PjRtValueTypeProto;
struct PjRtValueTypeProtoDefaultTypeInternal;
PROTOBUF_EXPORT extern PjRtValueTypeProtoDefaultTypeInternal _PjRtValueTypeProto_default_instance_;
class PjRtValueTypeProto_IntVector;
struct PjRtValueTypeProto_IntVectorDefaultTypeInternal;
PROTOBUF_EXPORT extern PjRtValueTypeProto_IntVectorDefaultTypeInternal _PjRtValueTypeProto_IntVector_default_instance_;
}  // namespace xla
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace xla {

// ===================================================================


// -------------------------------------------------------------------

class PROTOBUF_EXPORT PjRtValueTypeProto_IntVector final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:xla.PjRtValueTypeProto.IntVector) */ {
 public:
  inline PjRtValueTypeProto_IntVector() : PjRtValueTypeProto_IntVector(nullptr) {}
  ~PjRtValueTypeProto_IntVector() PROTOBUF_FINAL;
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PjRtValueTypeProto_IntVector(
      ::google::protobuf::internal::ConstantInitialized);

  inline PjRtValueTypeProto_IntVector(const PjRtValueTypeProto_IntVector& from) : PjRtValueTypeProto_IntVector(nullptr, from) {}
  inline PjRtValueTypeProto_IntVector(PjRtValueTypeProto_IntVector&& from) noexcept
      : PjRtValueTypeProto_IntVector(nullptr, std::move(from)) {}
  inline PjRtValueTypeProto_IntVector& operator=(const PjRtValueTypeProto_IntVector& from) {
    CopyFrom(from);
    return *this;
  }
  inline PjRtValueTypeProto_IntVector& operator=(PjRtValueTypeProto_IntVector&& from) noexcept {
    if (this == &from) return *this;
    if (GetArena() == from.GetArena()
#ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetArena() != nullptr
#endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PjRtValueTypeProto_IntVector& default_instance() {
    return *internal_default_instance();
  }
  static inline const PjRtValueTypeProto_IntVector* internal_default_instance() {
    return reinterpret_cast<const PjRtValueTypeProto_IntVector*>(
        &_PjRtValueTypeProto_IntVector_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(PjRtValueTypeProto_IntVector& a, PjRtValueTypeProto_IntVector& b) { a.Swap(&b); }
  inline void Swap(PjRtValueTypeProto_IntVector* other) {
    if (other == this) return;
#ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() != nullptr && GetArena() == other->GetArena()) {
#else   // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() == other->GetArena()) {
#endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PjRtValueTypeProto_IntVector* other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PjRtValueTypeProto_IntVector* New(::google::protobuf::Arena* arena = nullptr) const PROTOBUF_FINAL {
    return ::google::protobuf::Message::DefaultConstruct<PjRtValueTypeProto_IntVector>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PjRtValueTypeProto_IntVector& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PjRtValueTypeProto_IntVector& from) { PjRtValueTypeProto_IntVector::MergeImpl(*this, from); }

  private:
  static void MergeImpl(
      ::google::protobuf::MessageLite& to_msg,
      const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* _InternalSerialize(
      const MessageLite& msg, ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* arena);
  void SharedDtor();
  void InternalSwap(PjRtValueTypeProto_IntVector* other);
 private:
  friend class ::google::protobuf::internal::AnyMetadata;
  static ::absl::string_view FullMessageName() { return "xla.PjRtValueTypeProto.IntVector"; }

 protected:
  explicit PjRtValueTypeProto_IntVector(::google::protobuf::Arena* arena);
  PjRtValueTypeProto_IntVector(::google::protobuf::Arena* arena, const PjRtValueTypeProto_IntVector& from);
  PjRtValueTypeProto_IntVector(::google::protobuf::Arena* arena, PjRtValueTypeProto_IntVector&& from) noexcept
      : PjRtValueTypeProto_IntVector(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;

 public:
  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kValuesFieldNumber = 1,
  };
  // repeated int64 values = 1;
  int values_size() const;
  private:
  int _internal_values_size() const;

  public:
  void clear_values() ;
  ::int64_t values(int index) const;
  void set_values(int index, ::int64_t value);
  void add_values(::int64_t value);
  const ::google::protobuf::RepeatedField<::int64_t>& values() const;
  ::google::protobuf::RepeatedField<::int64_t>* mutable_values();

  private:
  const ::google::protobuf::RepeatedField<::int64_t>& _internal_values() const;
  ::google::protobuf::RepeatedField<::int64_t>* _internal_mutable_values();

  public:
  // @@protoc_insertion_point(class_scope:xla.PjRtValueTypeProto.IntVector)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      0, 1, 0,
      0, 2>
      _table_;


  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(
        ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena);
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena, const Impl_& from,
                          const PjRtValueTypeProto_IntVector& from_msg);
    ::google::protobuf::RepeatedField<::int64_t> values_;
    mutable ::google::protobuf::internal::CachedSize _values_cached_byte_size_;
    mutable ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fpjrt_2fproto_2fpjrt_5fvalue_5ftype_2eproto;
};
// -------------------------------------------------------------------

class PROTOBUF_EXPORT PjRtValueTypeProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:xla.PjRtValueTypeProto) */ {
 public:
  inline PjRtValueTypeProto() : PjRtValueTypeProto(nullptr) {}
  ~PjRtValueTypeProto() PROTOBUF_FINAL;
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PjRtValueTypeProto(
      ::google::protobuf::internal::ConstantInitialized);

  inline PjRtValueTypeProto(const PjRtValueTypeProto& from) : PjRtValueTypeProto(nullptr, from) {}
  inline PjRtValueTypeProto(PjRtValueTypeProto&& from) noexcept
      : PjRtValueTypeProto(nullptr, std::move(from)) {}
  inline PjRtValueTypeProto& operator=(const PjRtValueTypeProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline PjRtValueTypeProto& operator=(PjRtValueTypeProto&& from) noexcept {
    if (this == &from) return *this;
    if (GetArena() == from.GetArena()
#ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetArena() != nullptr
#endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PjRtValueTypeProto& default_instance() {
    return *internal_default_instance();
  }
  enum ValueCase {
    kStringValue = 1,
    kBoolValue = 2,
    kIntValue = 3,
    kIntVector = 4,
    kFloatValue = 5,
    VALUE_NOT_SET = 0,
  };
  static inline const PjRtValueTypeProto* internal_default_instance() {
    return reinterpret_cast<const PjRtValueTypeProto*>(
        &_PjRtValueTypeProto_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(PjRtValueTypeProto& a, PjRtValueTypeProto& b) { a.Swap(&b); }
  inline void Swap(PjRtValueTypeProto* other) {
    if (other == this) return;
#ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() != nullptr && GetArena() == other->GetArena()) {
#else   // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() == other->GetArena()) {
#endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PjRtValueTypeProto* other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PjRtValueTypeProto* New(::google::protobuf::Arena* arena = nullptr) const PROTOBUF_FINAL {
    return ::google::protobuf::Message::DefaultConstruct<PjRtValueTypeProto>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PjRtValueTypeProto& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PjRtValueTypeProto& from) { PjRtValueTypeProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(
      ::google::protobuf::MessageLite& to_msg,
      const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* _InternalSerialize(
      const MessageLite& msg, ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* arena);
  void SharedDtor();
  void InternalSwap(PjRtValueTypeProto* other);
 private:
  friend class ::google::protobuf::internal::AnyMetadata;
  static ::absl::string_view FullMessageName() { return "xla.PjRtValueTypeProto"; }

 protected:
  explicit PjRtValueTypeProto(::google::protobuf::Arena* arena);
  PjRtValueTypeProto(::google::protobuf::Arena* arena, const PjRtValueTypeProto& from);
  PjRtValueTypeProto(::google::protobuf::Arena* arena, PjRtValueTypeProto&& from) noexcept
      : PjRtValueTypeProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;

 public:
  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using IntVector = PjRtValueTypeProto_IntVector;

  // accessors -------------------------------------------------------
  enum : int {
    kStringValueFieldNumber = 1,
    kBoolValueFieldNumber = 2,
    kIntValueFieldNumber = 3,
    kIntVectorFieldNumber = 4,
    kFloatValueFieldNumber = 5,
  };
  // string string_value = 1;
  bool has_string_value() const;
  void clear_string_value() ;
  const std::string& string_value() const;
  template <typename Arg_ = const std::string&, typename... Args_>
  void set_string_value(Arg_&& arg, Args_... args);
  std::string* mutable_string_value();
  PROTOBUF_NODISCARD std::string* release_string_value();
  void set_allocated_string_value(std::string* value);

  private:
  const std::string& _internal_string_value() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_string_value(
      const std::string& value);
  std::string* _internal_mutable_string_value();

  public:
  // bool bool_value = 2;
  bool has_bool_value() const;
  void clear_bool_value() ;
  bool bool_value() const;
  void set_bool_value(bool value);

  private:
  bool _internal_bool_value() const;
  void _internal_set_bool_value(bool value);

  public:
  // int64 int_value = 3;
  bool has_int_value() const;
  void clear_int_value() ;
  ::int64_t int_value() const;
  void set_int_value(::int64_t value);

  private:
  ::int64_t _internal_int_value() const;
  void _internal_set_int_value(::int64_t value);

  public:
  // .xla.PjRtValueTypeProto.IntVector int_vector = 4;
  bool has_int_vector() const;
  private:
  bool _internal_has_int_vector() const;

  public:
  void clear_int_vector() ;
  const ::xla::PjRtValueTypeProto_IntVector& int_vector() const;
  PROTOBUF_NODISCARD ::xla::PjRtValueTypeProto_IntVector* release_int_vector();
  ::xla::PjRtValueTypeProto_IntVector* mutable_int_vector();
  void set_allocated_int_vector(::xla::PjRtValueTypeProto_IntVector* value);
  void unsafe_arena_set_allocated_int_vector(::xla::PjRtValueTypeProto_IntVector* value);
  ::xla::PjRtValueTypeProto_IntVector* unsafe_arena_release_int_vector();

  private:
  const ::xla::PjRtValueTypeProto_IntVector& _internal_int_vector() const;
  ::xla::PjRtValueTypeProto_IntVector* _internal_mutable_int_vector();

  public:
  // float float_value = 5;
  bool has_float_value() const;
  void clear_float_value() ;
  float float_value() const;
  void set_float_value(float value);

  private:
  float _internal_float_value() const;
  void _internal_set_float_value(float value);

  public:
  void clear_value();
  ValueCase value_case() const;
  // @@protoc_insertion_point(class_scope:xla.PjRtValueTypeProto)
 private:
  class _Internal;
  void set_has_string_value();
  void set_has_bool_value();
  void set_has_int_value();
  void set_has_int_vector();
  void set_has_float_value();
  inline bool has_value() const;
  inline void clear_has_value();
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      0, 5, 1,
      43, 2>
      _table_;


  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(
        ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena);
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena, const Impl_& from,
                          const PjRtValueTypeProto& from_msg);
    union ValueUnion {
      constexpr ValueUnion() : _constinit_{} {}
      ::google::protobuf::internal::ConstantInitialized _constinit_;
      ::google::protobuf::internal::ArenaStringPtr string_value_;
      bool bool_value_;
      ::int64_t int_value_;
      ::xla::PjRtValueTypeProto_IntVector* int_vector_;
      float float_value_;
    } value_;
    mutable ::google::protobuf::internal::CachedSize _cached_size_;
    ::uint32_t _oneof_case_[1];
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fpjrt_2fproto_2fpjrt_5fvalue_5ftype_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// PjRtValueTypeProto_IntVector

// repeated int64 values = 1;
inline int PjRtValueTypeProto_IntVector::_internal_values_size() const {
  return _internal_values().size();
}
inline int PjRtValueTypeProto_IntVector::values_size() const {
  return _internal_values_size();
}
inline void PjRtValueTypeProto_IntVector::clear_values() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.values_.Clear();
}
inline ::int64_t PjRtValueTypeProto_IntVector::values(int index) const {
  // @@protoc_insertion_point(field_get:xla.PjRtValueTypeProto.IntVector.values)
  return _internal_values().Get(index);
}
inline void PjRtValueTypeProto_IntVector::set_values(int index, ::int64_t value) {
  _internal_mutable_values()->Set(index, value);
  // @@protoc_insertion_point(field_set:xla.PjRtValueTypeProto.IntVector.values)
}
inline void PjRtValueTypeProto_IntVector::add_values(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_values()->Add(value);
  // @@protoc_insertion_point(field_add:xla.PjRtValueTypeProto.IntVector.values)
}
inline const ::google::protobuf::RepeatedField<::int64_t>& PjRtValueTypeProto_IntVector::values() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:xla.PjRtValueTypeProto.IntVector.values)
  return _internal_values();
}
inline ::google::protobuf::RepeatedField<::int64_t>* PjRtValueTypeProto_IntVector::mutable_values()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:xla.PjRtValueTypeProto.IntVector.values)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_values();
}
inline const ::google::protobuf::RepeatedField<::int64_t>&
PjRtValueTypeProto_IntVector::_internal_values() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.values_;
}
inline ::google::protobuf::RepeatedField<::int64_t>* PjRtValueTypeProto_IntVector::_internal_mutable_values() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.values_;
}

// -------------------------------------------------------------------

// PjRtValueTypeProto

// string string_value = 1;
inline bool PjRtValueTypeProto::has_string_value() const {
  return value_case() == kStringValue;
}
inline void PjRtValueTypeProto::set_has_string_value() {
  _impl_._oneof_case_[0] = kStringValue;
}
inline void PjRtValueTypeProto::clear_string_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value_case() == kStringValue) {
    _impl_.value_.string_value_.Destroy();
    clear_has_value();
  }
}
inline const std::string& PjRtValueTypeProto::string_value() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:xla.PjRtValueTypeProto.string_value)
  return _internal_string_value();
}
template <typename Arg_, typename... Args_>
inline PROTOBUF_ALWAYS_INLINE void PjRtValueTypeProto::set_string_value(Arg_&& arg,
                                                     Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value_case() != kStringValue) {
    clear_value();

    set_has_string_value();
    _impl_.value_.string_value_.InitDefault();
  }
  _impl_.value_.string_value_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:xla.PjRtValueTypeProto.string_value)
}
inline std::string* PjRtValueTypeProto::mutable_string_value() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  std::string* _s = _internal_mutable_string_value();
  // @@protoc_insertion_point(field_mutable:xla.PjRtValueTypeProto.string_value)
  return _s;
}
inline const std::string& PjRtValueTypeProto::_internal_string_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  if (value_case() != kStringValue) {
    return ::google::protobuf::internal::GetEmptyStringAlreadyInited();
  }
  return _impl_.value_.string_value_.Get();
}
inline void PjRtValueTypeProto::_internal_set_string_value(const std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value_case() != kStringValue) {
    clear_value();

    set_has_string_value();
    _impl_.value_.string_value_.InitDefault();
  }
  _impl_.value_.string_value_.Set(value, GetArena());
}
inline std::string* PjRtValueTypeProto::_internal_mutable_string_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value_case() != kStringValue) {
    clear_value();

    set_has_string_value();
    _impl_.value_.string_value_.InitDefault();
  }
  return _impl_.value_.string_value_.Mutable( GetArena());
}
inline std::string* PjRtValueTypeProto::release_string_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:xla.PjRtValueTypeProto.string_value)
  if (value_case() != kStringValue) {
    return nullptr;
  }
  clear_has_value();
  return _impl_.value_.string_value_.Release();
}
inline void PjRtValueTypeProto::set_allocated_string_value(std::string* value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (has_value()) {
    clear_value();
  }
  if (value != nullptr) {
    set_has_string_value();
    _impl_.value_.string_value_.InitAllocated(value, GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:xla.PjRtValueTypeProto.string_value)
}

// bool bool_value = 2;
inline bool PjRtValueTypeProto::has_bool_value() const {
  return value_case() == kBoolValue;
}
inline void PjRtValueTypeProto::set_has_bool_value() {
  _impl_._oneof_case_[0] = kBoolValue;
}
inline void PjRtValueTypeProto::clear_bool_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value_case() == kBoolValue) {
    _impl_.value_.bool_value_ = false;
    clear_has_value();
  }
}
inline bool PjRtValueTypeProto::bool_value() const {
  // @@protoc_insertion_point(field_get:xla.PjRtValueTypeProto.bool_value)
  return _internal_bool_value();
}
inline void PjRtValueTypeProto::set_bool_value(bool value) {
  if (value_case() != kBoolValue) {
    clear_value();
    set_has_bool_value();
  }
  _impl_.value_.bool_value_ = value;
  // @@protoc_insertion_point(field_set:xla.PjRtValueTypeProto.bool_value)
}
inline bool PjRtValueTypeProto::_internal_bool_value() const {
  if (value_case() == kBoolValue) {
    return _impl_.value_.bool_value_;
  }
  return false;
}

// int64 int_value = 3;
inline bool PjRtValueTypeProto::has_int_value() const {
  return value_case() == kIntValue;
}
inline void PjRtValueTypeProto::set_has_int_value() {
  _impl_._oneof_case_[0] = kIntValue;
}
inline void PjRtValueTypeProto::clear_int_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value_case() == kIntValue) {
    _impl_.value_.int_value_ = ::int64_t{0};
    clear_has_value();
  }
}
inline ::int64_t PjRtValueTypeProto::int_value() const {
  // @@protoc_insertion_point(field_get:xla.PjRtValueTypeProto.int_value)
  return _internal_int_value();
}
inline void PjRtValueTypeProto::set_int_value(::int64_t value) {
  if (value_case() != kIntValue) {
    clear_value();
    set_has_int_value();
  }
  _impl_.value_.int_value_ = value;
  // @@protoc_insertion_point(field_set:xla.PjRtValueTypeProto.int_value)
}
inline ::int64_t PjRtValueTypeProto::_internal_int_value() const {
  if (value_case() == kIntValue) {
    return _impl_.value_.int_value_;
  }
  return ::int64_t{0};
}

// .xla.PjRtValueTypeProto.IntVector int_vector = 4;
inline bool PjRtValueTypeProto::has_int_vector() const {
  return value_case() == kIntVector;
}
inline bool PjRtValueTypeProto::_internal_has_int_vector() const {
  return value_case() == kIntVector;
}
inline void PjRtValueTypeProto::set_has_int_vector() {
  _impl_._oneof_case_[0] = kIntVector;
}
inline void PjRtValueTypeProto::clear_int_vector() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value_case() == kIntVector) {
    if (GetArena() == nullptr) {
      delete _impl_.value_.int_vector_;
    } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
      ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.value_.int_vector_);
    }
    clear_has_value();
  }
}
inline ::xla::PjRtValueTypeProto_IntVector* PjRtValueTypeProto::release_int_vector() {
  // @@protoc_insertion_point(field_release:xla.PjRtValueTypeProto.int_vector)
  if (value_case() == kIntVector) {
    clear_has_value();
    auto* temp = _impl_.value_.int_vector_;
    if (GetArena() != nullptr) {
      temp = ::google::protobuf::internal::DuplicateIfNonNull(temp);
    }
    _impl_.value_.int_vector_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::xla::PjRtValueTypeProto_IntVector& PjRtValueTypeProto::_internal_int_vector() const {
  return value_case() == kIntVector ? *_impl_.value_.int_vector_ : reinterpret_cast<::xla::PjRtValueTypeProto_IntVector&>(::xla::_PjRtValueTypeProto_IntVector_default_instance_);
}
inline const ::xla::PjRtValueTypeProto_IntVector& PjRtValueTypeProto::int_vector() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:xla.PjRtValueTypeProto.int_vector)
  return _internal_int_vector();
}
inline ::xla::PjRtValueTypeProto_IntVector* PjRtValueTypeProto::unsafe_arena_release_int_vector() {
  // @@protoc_insertion_point(field_unsafe_arena_release:xla.PjRtValueTypeProto.int_vector)
  if (value_case() == kIntVector) {
    clear_has_value();
    auto* temp = _impl_.value_.int_vector_;
    _impl_.value_.int_vector_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void PjRtValueTypeProto::unsafe_arena_set_allocated_int_vector(::xla::PjRtValueTypeProto_IntVector* value) {
  // We rely on the oneof clear method to free the earlier contents
  // of this oneof. We can directly use the pointer we're given to
  // set the new value.
  clear_value();
  if (value) {
    set_has_int_vector();
    _impl_.value_.int_vector_ = value;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:xla.PjRtValueTypeProto.int_vector)
}
inline ::xla::PjRtValueTypeProto_IntVector* PjRtValueTypeProto::_internal_mutable_int_vector() {
  if (value_case() != kIntVector) {
    clear_value();
    set_has_int_vector();
    _impl_.value_.int_vector_ =
        ::google::protobuf::Message::DefaultConstruct<::xla::PjRtValueTypeProto_IntVector>(GetArena());
  }
  return _impl_.value_.int_vector_;
}
inline ::xla::PjRtValueTypeProto_IntVector* PjRtValueTypeProto::mutable_int_vector() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::xla::PjRtValueTypeProto_IntVector* _msg = _internal_mutable_int_vector();
  // @@protoc_insertion_point(field_mutable:xla.PjRtValueTypeProto.int_vector)
  return _msg;
}

// float float_value = 5;
inline bool PjRtValueTypeProto::has_float_value() const {
  return value_case() == kFloatValue;
}
inline void PjRtValueTypeProto::set_has_float_value() {
  _impl_._oneof_case_[0] = kFloatValue;
}
inline void PjRtValueTypeProto::clear_float_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value_case() == kFloatValue) {
    _impl_.value_.float_value_ = 0;
    clear_has_value();
  }
}
inline float PjRtValueTypeProto::float_value() const {
  // @@protoc_insertion_point(field_get:xla.PjRtValueTypeProto.float_value)
  return _internal_float_value();
}
inline void PjRtValueTypeProto::set_float_value(float value) {
  if (value_case() != kFloatValue) {
    clear_value();
    set_has_float_value();
  }
  _impl_.value_.float_value_ = value;
  // @@protoc_insertion_point(field_set:xla.PjRtValueTypeProto.float_value)
}
inline float PjRtValueTypeProto::_internal_float_value() const {
  if (value_case() == kFloatValue) {
    return _impl_.value_.float_value_;
  }
  return 0;
}

inline bool PjRtValueTypeProto::has_value() const {
  return value_case() != VALUE_NOT_SET;
}
inline void PjRtValueTypeProto::clear_has_value() {
  _impl_._oneof_case_[0] = VALUE_NOT_SET;
}
inline PjRtValueTypeProto::ValueCase PjRtValueTypeProto::value_case() const {
  return PjRtValueTypeProto::ValueCase(_impl_._oneof_case_[0]);
}
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace xla


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // GOOGLE_PROTOBUF_INCLUDED_xla_2fpjrt_2fproto_2fpjrt_5fvalue_5ftype_2eproto_2epb_2eh
