//===- RelayoutOpInterface.h - Relayout op interface ----------------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//
//
// This file implements the operation interface for relayout ops.
//
//===----------------------------------------------------------------------===//

#ifndef MLIR_DIALECT_RELAYOUTOPINTERFACE_H_
#define MLIR_DIALECT_RELAYOUTOPINTERFACE_H_

#include "mlir/IR/OpDefinition.h"

#include "mlir/Dialect/Linalg/IR/RelayoutOpInterface.h.inc"

#endif // MLIR_DIALECT_RELAYOUTOPINTERFACE_H_
