# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.nn.rnn_cell namespace
"""

import sys as _sys

from tensorflow.python.keras.layers.legacy_rnn.rnn_cell_impl import BasicLSTMCell # line: 654
from tensorflow.python.keras.layers.legacy_rnn.rnn_cell_impl import BasicRNNCell # line: 399
from tensorflow.python.keras.layers.legacy_rnn.rnn_cell_impl import DeviceWrapper # line: 1213
from tensorflow.python.keras.layers.legacy_rnn.rnn_cell_impl import DropoutWrapper # line: 1191
from tensorflow.python.keras.layers.legacy_rnn.rnn_cell_impl import GRUCell # line: 495
from tensorflow.python.keras.layers.legacy_rnn.rnn_cell_impl import LSTMCell # line: 824
from tensorflow.python.keras.layers.legacy_rnn.rnn_cell_impl import LSTMStateTuple # line: 634
from tensorflow.python.keras.layers.legacy_rnn.rnn_cell_impl import MultiRNNCell # line: 1223
from tensorflow.python.keras.layers.legacy_rnn.rnn_cell_impl import RNNCell # line: 184
from tensorflow.python.keras.layers.legacy_rnn.rnn_cell_impl import ResidualWrapper # line: 1202

from tensorflow.python.util import module_wrapper as _module_wrapper

if not isinstance(_sys.modules[__name__], _module_wrapper.TFModuleWrapper):
  _sys.modules[__name__] = _module_wrapper.TFModuleWrapper(
      _sys.modules[__name__], "nn.rnn_cell", public_apis=None, deprecation=False,
      has_lite=False)
