# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.autograph namespace
"""

import sys as _sys

from tensorflow._api.v2.compat.v2.autograph import experimental
from tensorflow.python.autograph.impl.api import to_code # line: 904
from tensorflow.python.autograph.impl.api import to_graph # line: 707
from tensorflow.python.autograph.utils.ag_logging import set_verbosity # line: 36
from tensorflow.python.autograph.utils.ag_logging import trace # line: 87
