/* Copyright 2024 The OpenXLA Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================*/

#ifndef XLA_SERVICE_CPU_METRICS_H_
#define XLA_SERVICE_CPU_METRICS_H_

#include "absl/strings/string_view.h"

namespace xla {
namespace cpu {

// Records the stacktrace of the CPU compiler.
void RecordCpuCompilerStacktrace();

// Returns the number of times the GPU compiler was called with the given
// stacktrace.
int GetCpuCompilerStacktraceCount(absl::string_view stacktrace);

}  // namespace cpu
}  // namespace xla

#endif  // XLA_SERVICE_CPU_METRICS_H_
