# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.distribute.experimental namespace
"""

import sys as _sys

from tensorflow.python.distribute.central_storage_strategy import CentralStorageStrategyV1 as CentralStorageStrategy # line: 212
from tensorflow.python.distribute.collective_all_reduce_strategy import CollectiveAllReduceStrategyV1 as MultiWorkerMirroredStrategy # line: 290
from tensorflow.python.distribute.collective_util import CommunicationImplementation as CollectiveCommunication # line: 26
from tensorflow.python.distribute.collective_util import Hints as CollectiveHints # line: 168
from tensorflow.python.distribute.collective_util import CommunicationImplementation # line: 26
from tensorflow.python.distribute.collective_util import _OptionsExported as CommunicationOptions # line: 50
from tensorflow.python.distribute.parameter_server_strategy import ParameterServerStrategyV1 as ParameterServerStrategy # line: 48
from tensorflow.python.distribute.tpu_strategy import TPUStrategyV1 as TPUStrategy # line: 752

from tensorflow.python.util import module_wrapper as _module_wrapper

if not isinstance(_sys.modules[__name__], _module_wrapper.TFModuleWrapper):
  _sys.modules[__name__] = _module_wrapper.TFModuleWrapper(
      _sys.modules[__name__], "distribute.experimental", public_apis=None, deprecation=False,
      has_lite=False)
