# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.__internal__.tf2 namespace
"""

import sys as _sys

from tensorflow.python.tf2 import enabled # line: 35
