# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.__internal__.ops namespace
"""

import sys as _sys

from tensorflow.python.ops.handle_data_util import get_resource_handle_data # line: 25
from tensorflow.python.ops.resource_variable_ops import is_resource_variable # line: 2566
from tensorflow.python.ops.weights_broadcast_ops import broadcast_weights # line: 136
