// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: xla/pjrt/proto/execute_options.proto
// Protobuf C++ Version: 5.28.3

#ifndef GOOGLE_PROTOBUF_INCLUDED_xla_2fpjrt_2fproto_2fexecute_5foptions_2eproto_2epb_2eh
#define GOOGLE_PROTOBUF_INCLUDED_xla_2fpjrt_2fproto_2fexecute_5foptions_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 5028003
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_xla_2fpjrt_2fproto_2fexecute_5foptions_2eproto PROTOBUF_EXPORT

namespace google {
namespace protobuf {
namespace internal {
class AnyMetadata;
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct PROTOBUF_EXPORT TableStruct_xla_2fpjrt_2fproto_2fexecute_5foptions_2eproto {
  static const ::uint32_t offsets[];
};
PROTOBUF_EXPORT extern const ::google::protobuf::internal::DescriptorTable
    descriptor_table_xla_2fpjrt_2fproto_2fexecute_5foptions_2eproto;
namespace xla {
class ExecuteOptionsProto;
struct ExecuteOptionsProtoDefaultTypeInternal;
PROTOBUF_EXPORT extern ExecuteOptionsProtoDefaultTypeInternal _ExecuteOptionsProto_default_instance_;
}  // namespace xla
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace xla {
enum ExecutionModeProto : int {
  EXECUTION_MODE_UNSPECIFIED = 0,
  EXECUTION_MODE_DEFAULT = 1,
  EXECUTION_MODE_SYNCHRONOUS = 2,
  EXECUTION_MODE_ASYNCHRONOUS = 3,
  ExecutionModeProto_INT_MIN_SENTINEL_DO_NOT_USE_ =
      std::numeric_limits<::int32_t>::min(),
  ExecutionModeProto_INT_MAX_SENTINEL_DO_NOT_USE_ =
      std::numeric_limits<::int32_t>::max(),
};

PROTOBUF_EXPORT bool ExecutionModeProto_IsValid(int value);
PROTOBUF_EXPORT extern const uint32_t ExecutionModeProto_internal_data_[];
constexpr ExecutionModeProto ExecutionModeProto_MIN = static_cast<ExecutionModeProto>(0);
constexpr ExecutionModeProto ExecutionModeProto_MAX = static_cast<ExecutionModeProto>(3);
constexpr int ExecutionModeProto_ARRAYSIZE = 3 + 1;
PROTOBUF_EXPORT const ::google::protobuf::EnumDescriptor*
ExecutionModeProto_descriptor();
template <typename T>
const std::string& ExecutionModeProto_Name(T value) {
  static_assert(std::is_same<T, ExecutionModeProto>::value ||
                    std::is_integral<T>::value,
                "Incorrect type passed to ExecutionModeProto_Name().");
  return ExecutionModeProto_Name(static_cast<ExecutionModeProto>(value));
}
template <>
inline const std::string& ExecutionModeProto_Name(ExecutionModeProto value) {
  return ::google::protobuf::internal::NameOfDenseEnum<ExecutionModeProto_descriptor,
                                                 0, 3>(
      static_cast<int>(value));
}
inline bool ExecutionModeProto_Parse(absl::string_view name, ExecutionModeProto* value) {
  return ::google::protobuf::internal::ParseNamedEnum<ExecutionModeProto>(
      ExecutionModeProto_descriptor(), name, value);
}

// ===================================================================


// -------------------------------------------------------------------

class PROTOBUF_EXPORT ExecuteOptionsProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:xla.ExecuteOptionsProto) */ {
 public:
  inline ExecuteOptionsProto() : ExecuteOptionsProto(nullptr) {}
  ~ExecuteOptionsProto() PROTOBUF_FINAL;
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ExecuteOptionsProto(
      ::google::protobuf::internal::ConstantInitialized);

  inline ExecuteOptionsProto(const ExecuteOptionsProto& from) : ExecuteOptionsProto(nullptr, from) {}
  inline ExecuteOptionsProto(ExecuteOptionsProto&& from) noexcept
      : ExecuteOptionsProto(nullptr, std::move(from)) {}
  inline ExecuteOptionsProto& operator=(const ExecuteOptionsProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExecuteOptionsProto& operator=(ExecuteOptionsProto&& from) noexcept {
    if (this == &from) return *this;
    if (GetArena() == from.GetArena()
#ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetArena() != nullptr
#endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ExecuteOptionsProto& default_instance() {
    return *internal_default_instance();
  }
  static inline const ExecuteOptionsProto* internal_default_instance() {
    return reinterpret_cast<const ExecuteOptionsProto*>(
        &_ExecuteOptionsProto_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(ExecuteOptionsProto& a, ExecuteOptionsProto& b) { a.Swap(&b); }
  inline void Swap(ExecuteOptionsProto* other) {
    if (other == this) return;
#ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() != nullptr && GetArena() == other->GetArena()) {
#else   // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() == other->GetArena()) {
#endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExecuteOptionsProto* other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ExecuteOptionsProto* New(::google::protobuf::Arena* arena = nullptr) const PROTOBUF_FINAL {
    return ::google::protobuf::Message::DefaultConstruct<ExecuteOptionsProto>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExecuteOptionsProto& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExecuteOptionsProto& from) { ExecuteOptionsProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(
      ::google::protobuf::MessageLite& to_msg,
      const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* _InternalSerialize(
      const MessageLite& msg, ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* arena);
  void SharedDtor();
  void InternalSwap(ExecuteOptionsProto* other);
 private:
  friend class ::google::protobuf::internal::AnyMetadata;
  static ::absl::string_view FullMessageName() { return "xla.ExecuteOptionsProto"; }

 protected:
  explicit ExecuteOptionsProto(::google::protobuf::Arena* arena);
  ExecuteOptionsProto(::google::protobuf::Arena* arena, const ExecuteOptionsProto& from);
  ExecuteOptionsProto(::google::protobuf::Arena* arena, ExecuteOptionsProto&& from) noexcept
      : ExecuteOptionsProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;

 public:
  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNonDonatableInputIndicesFieldNumber = 7,
    kLaunchIdFieldNumber = 3,
    kArgumentsAreTupledFieldNumber = 1,
    kUntupleResultFieldNumber = 2,
    kStrictShapeCheckingFieldNumber = 4,
    kUseMajorToMinorDataLayoutForCallbacksFieldNumber = 8,
    kExecutionModeFieldNumber = 6,
  };
  // repeated int32 non_donatable_input_indices = 7;
  int non_donatable_input_indices_size() const;
  private:
  int _internal_non_donatable_input_indices_size() const;

  public:
  void clear_non_donatable_input_indices() ;
  ::int32_t non_donatable_input_indices(int index) const;
  void set_non_donatable_input_indices(int index, ::int32_t value);
  void add_non_donatable_input_indices(::int32_t value);
  const ::google::protobuf::RepeatedField<::int32_t>& non_donatable_input_indices() const;
  ::google::protobuf::RepeatedField<::int32_t>* mutable_non_donatable_input_indices();

  private:
  const ::google::protobuf::RepeatedField<::int32_t>& _internal_non_donatable_input_indices() const;
  ::google::protobuf::RepeatedField<::int32_t>* _internal_mutable_non_donatable_input_indices();

  public:
  // int32 launch_id = 3;
  void clear_launch_id() ;
  ::int32_t launch_id() const;
  void set_launch_id(::int32_t value);

  private:
  ::int32_t _internal_launch_id() const;
  void _internal_set_launch_id(::int32_t value);

  public:
  // bool arguments_are_tupled = 1;
  void clear_arguments_are_tupled() ;
  bool arguments_are_tupled() const;
  void set_arguments_are_tupled(bool value);

  private:
  bool _internal_arguments_are_tupled() const;
  void _internal_set_arguments_are_tupled(bool value);

  public:
  // bool untuple_result = 2;
  void clear_untuple_result() ;
  bool untuple_result() const;
  void set_untuple_result(bool value);

  private:
  bool _internal_untuple_result() const;
  void _internal_set_untuple_result(bool value);

  public:
  // bool strict_shape_checking = 4;
  void clear_strict_shape_checking() ;
  bool strict_shape_checking() const;
  void set_strict_shape_checking(bool value);

  private:
  bool _internal_strict_shape_checking() const;
  void _internal_set_strict_shape_checking(bool value);

  public:
  // bool use_major_to_minor_data_layout_for_callbacks = 8;
  void clear_use_major_to_minor_data_layout_for_callbacks() ;
  bool use_major_to_minor_data_layout_for_callbacks() const;
  void set_use_major_to_minor_data_layout_for_callbacks(bool value);

  private:
  bool _internal_use_major_to_minor_data_layout_for_callbacks() const;
  void _internal_set_use_major_to_minor_data_layout_for_callbacks(bool value);

  public:
  // .xla.ExecutionModeProto execution_mode = 6;
  void clear_execution_mode() ;
  ::xla::ExecutionModeProto execution_mode() const;
  void set_execution_mode(::xla::ExecutionModeProto value);

  private:
  ::xla::ExecutionModeProto _internal_execution_mode() const;
  void _internal_set_execution_mode(::xla::ExecutionModeProto value);

  public:
  // @@protoc_insertion_point(class_scope:xla.ExecuteOptionsProto)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      3, 7, 0,
      0, 2>
      _table_;


  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(
        ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena);
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena, const Impl_& from,
                          const ExecuteOptionsProto& from_msg);
    ::google::protobuf::RepeatedField<::int32_t> non_donatable_input_indices_;
    mutable ::google::protobuf::internal::CachedSize _non_donatable_input_indices_cached_byte_size_;
    ::int32_t launch_id_;
    bool arguments_are_tupled_;
    bool untuple_result_;
    bool strict_shape_checking_;
    bool use_major_to_minor_data_layout_for_callbacks_;
    int execution_mode_;
    mutable ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fpjrt_2fproto_2fexecute_5foptions_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// ExecuteOptionsProto

// bool arguments_are_tupled = 1;
inline void ExecuteOptionsProto::clear_arguments_are_tupled() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arguments_are_tupled_ = false;
}
inline bool ExecuteOptionsProto::arguments_are_tupled() const {
  // @@protoc_insertion_point(field_get:xla.ExecuteOptionsProto.arguments_are_tupled)
  return _internal_arguments_are_tupled();
}
inline void ExecuteOptionsProto::set_arguments_are_tupled(bool value) {
  _internal_set_arguments_are_tupled(value);
  // @@protoc_insertion_point(field_set:xla.ExecuteOptionsProto.arguments_are_tupled)
}
inline bool ExecuteOptionsProto::_internal_arguments_are_tupled() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.arguments_are_tupled_;
}
inline void ExecuteOptionsProto::_internal_set_arguments_are_tupled(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arguments_are_tupled_ = value;
}

// bool untuple_result = 2;
inline void ExecuteOptionsProto::clear_untuple_result() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.untuple_result_ = false;
}
inline bool ExecuteOptionsProto::untuple_result() const {
  // @@protoc_insertion_point(field_get:xla.ExecuteOptionsProto.untuple_result)
  return _internal_untuple_result();
}
inline void ExecuteOptionsProto::set_untuple_result(bool value) {
  _internal_set_untuple_result(value);
  // @@protoc_insertion_point(field_set:xla.ExecuteOptionsProto.untuple_result)
}
inline bool ExecuteOptionsProto::_internal_untuple_result() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.untuple_result_;
}
inline void ExecuteOptionsProto::_internal_set_untuple_result(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.untuple_result_ = value;
}

// int32 launch_id = 3;
inline void ExecuteOptionsProto::clear_launch_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.launch_id_ = 0;
}
inline ::int32_t ExecuteOptionsProto::launch_id() const {
  // @@protoc_insertion_point(field_get:xla.ExecuteOptionsProto.launch_id)
  return _internal_launch_id();
}
inline void ExecuteOptionsProto::set_launch_id(::int32_t value) {
  _internal_set_launch_id(value);
  // @@protoc_insertion_point(field_set:xla.ExecuteOptionsProto.launch_id)
}
inline ::int32_t ExecuteOptionsProto::_internal_launch_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.launch_id_;
}
inline void ExecuteOptionsProto::_internal_set_launch_id(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.launch_id_ = value;
}

// bool strict_shape_checking = 4;
inline void ExecuteOptionsProto::clear_strict_shape_checking() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.strict_shape_checking_ = false;
}
inline bool ExecuteOptionsProto::strict_shape_checking() const {
  // @@protoc_insertion_point(field_get:xla.ExecuteOptionsProto.strict_shape_checking)
  return _internal_strict_shape_checking();
}
inline void ExecuteOptionsProto::set_strict_shape_checking(bool value) {
  _internal_set_strict_shape_checking(value);
  // @@protoc_insertion_point(field_set:xla.ExecuteOptionsProto.strict_shape_checking)
}
inline bool ExecuteOptionsProto::_internal_strict_shape_checking() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.strict_shape_checking_;
}
inline void ExecuteOptionsProto::_internal_set_strict_shape_checking(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.strict_shape_checking_ = value;
}

// bool use_major_to_minor_data_layout_for_callbacks = 8;
inline void ExecuteOptionsProto::clear_use_major_to_minor_data_layout_for_callbacks() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.use_major_to_minor_data_layout_for_callbacks_ = false;
}
inline bool ExecuteOptionsProto::use_major_to_minor_data_layout_for_callbacks() const {
  // @@protoc_insertion_point(field_get:xla.ExecuteOptionsProto.use_major_to_minor_data_layout_for_callbacks)
  return _internal_use_major_to_minor_data_layout_for_callbacks();
}
inline void ExecuteOptionsProto::set_use_major_to_minor_data_layout_for_callbacks(bool value) {
  _internal_set_use_major_to_minor_data_layout_for_callbacks(value);
  // @@protoc_insertion_point(field_set:xla.ExecuteOptionsProto.use_major_to_minor_data_layout_for_callbacks)
}
inline bool ExecuteOptionsProto::_internal_use_major_to_minor_data_layout_for_callbacks() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.use_major_to_minor_data_layout_for_callbacks_;
}
inline void ExecuteOptionsProto::_internal_set_use_major_to_minor_data_layout_for_callbacks(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.use_major_to_minor_data_layout_for_callbacks_ = value;
}

// .xla.ExecutionModeProto execution_mode = 6;
inline void ExecuteOptionsProto::clear_execution_mode() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.execution_mode_ = 0;
}
inline ::xla::ExecutionModeProto ExecuteOptionsProto::execution_mode() const {
  // @@protoc_insertion_point(field_get:xla.ExecuteOptionsProto.execution_mode)
  return _internal_execution_mode();
}
inline void ExecuteOptionsProto::set_execution_mode(::xla::ExecutionModeProto value) {
  _internal_set_execution_mode(value);
  // @@protoc_insertion_point(field_set:xla.ExecuteOptionsProto.execution_mode)
}
inline ::xla::ExecutionModeProto ExecuteOptionsProto::_internal_execution_mode() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::xla::ExecutionModeProto>(_impl_.execution_mode_);
}
inline void ExecuteOptionsProto::_internal_set_execution_mode(::xla::ExecutionModeProto value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.execution_mode_ = value;
}

// repeated int32 non_donatable_input_indices = 7;
inline int ExecuteOptionsProto::_internal_non_donatable_input_indices_size() const {
  return _internal_non_donatable_input_indices().size();
}
inline int ExecuteOptionsProto::non_donatable_input_indices_size() const {
  return _internal_non_donatable_input_indices_size();
}
inline void ExecuteOptionsProto::clear_non_donatable_input_indices() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.non_donatable_input_indices_.Clear();
}
inline ::int32_t ExecuteOptionsProto::non_donatable_input_indices(int index) const {
  // @@protoc_insertion_point(field_get:xla.ExecuteOptionsProto.non_donatable_input_indices)
  return _internal_non_donatable_input_indices().Get(index);
}
inline void ExecuteOptionsProto::set_non_donatable_input_indices(int index, ::int32_t value) {
  _internal_mutable_non_donatable_input_indices()->Set(index, value);
  // @@protoc_insertion_point(field_set:xla.ExecuteOptionsProto.non_donatable_input_indices)
}
inline void ExecuteOptionsProto::add_non_donatable_input_indices(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_non_donatable_input_indices()->Add(value);
  // @@protoc_insertion_point(field_add:xla.ExecuteOptionsProto.non_donatable_input_indices)
}
inline const ::google::protobuf::RepeatedField<::int32_t>& ExecuteOptionsProto::non_donatable_input_indices() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:xla.ExecuteOptionsProto.non_donatable_input_indices)
  return _internal_non_donatable_input_indices();
}
inline ::google::protobuf::RepeatedField<::int32_t>* ExecuteOptionsProto::mutable_non_donatable_input_indices()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:xla.ExecuteOptionsProto.non_donatable_input_indices)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_non_donatable_input_indices();
}
inline const ::google::protobuf::RepeatedField<::int32_t>&
ExecuteOptionsProto::_internal_non_donatable_input_indices() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.non_donatable_input_indices_;
}
inline ::google::protobuf::RepeatedField<::int32_t>* ExecuteOptionsProto::_internal_mutable_non_donatable_input_indices() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.non_donatable_input_indices_;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace xla


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::xla::ExecutionModeProto> : std::true_type {};
template <>
inline const EnumDescriptor* GetEnumDescriptor<::xla::ExecutionModeProto>() {
  return ::xla::ExecutionModeProto_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // GOOGLE_PROTOBUF_INCLUDED_xla_2fpjrt_2fproto_2fexecute_5foptions_2eproto_2epb_2eh
