# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.autograph.experimental namespace
"""

import sys as _sys

from tensorflow.python.autograph.core.converter import Feature # line: 78
from tensorflow.python.autograph.impl.api import do_not_convert # line: 624
from tensorflow.python.autograph.lang.directives import set_loop_options # line: 45
