# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.lite.constants namespace
"""

import sys as _sys

from tensorflow.lite.python.lite_constants import FLOAT # line: 50
from tensorflow.lite.python.lite_constants import FLOAT16 # line: 51
from tensorflow.lite.python.lite_constants import GRAPHVIZ_DOT # line: 60
from tensorflow.lite.python.lite_constants import INT16 # line: 58
from tensorflow.lite.python.lite_constants import INT32 # line: 52
from tensorflow.lite.python.lite_constants import INT64 # line: 53
from tensorflow.lite.python.lite_constants import INT8 # line: 57
from tensorflow.lite.python.lite_constants import QUANTIZED_UINT8 # line: 55
from tensorflow.lite.python.lite_constants import STRING # line: 54
from tensorflow.lite.python.lite_constants import TFLITE # line: 59

from tensorflow.python.util import module_wrapper as _module_wrapper

if not isinstance(_sys.modules[__name__], _module_wrapper.TFModuleWrapper):
  _sys.modules[__name__] = _module_wrapper.TFModuleWrapper(
      _sys.modules[__name__], "lite.constants", public_apis=None, deprecation=False,
      has_lite=False)
