# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.__internal__ namespace
"""

import sys as _sys

from tensorflow._api.v2.compat.v2.__internal__ import autograph
from tensorflow._api.v2.compat.v2.__internal__ import decorator
from tensorflow._api.v2.compat.v2.__internal__ import dispatch
from tensorflow._api.v2.compat.v2.__internal__ import distribute
from tensorflow._api.v2.compat.v2.__internal__ import eager_context
from tensorflow._api.v2.compat.v2.__internal__ import feature_column
from tensorflow._api.v2.compat.v2.__internal__ import function
from tensorflow._api.v2.compat.v2.__internal__ import graph_util
from tensorflow._api.v2.compat.v2.__internal__ import mixed_precision
from tensorflow._api.v2.compat.v2.__internal__ import monitoring
from tensorflow._api.v2.compat.v2.__internal__ import nest
from tensorflow._api.v2.compat.v2.__internal__ import ops
from tensorflow._api.v2.compat.v2.__internal__ import saved_model
from tensorflow._api.v2.compat.v2.__internal__ import smart_cond
from tensorflow._api.v2.compat.v2.__internal__ import test
from tensorflow._api.v2.compat.v2.__internal__ import tf2
from tensorflow._api.v2.compat.v2.__internal__ import tracking
from tensorflow._api.v2.compat.v2.__internal__ import train
from tensorflow._api.v2.compat.v2.__internal__ import types
from tensorflow.python.distribute.load_context import load_context # line: 49
from tensorflow.python.eager.backprop import record_gradient # line: 160
from tensorflow.python.eager.context import is_tfrt_enabled # line: 215
from tensorflow.python.eager.lift_to_graph import lift_to_graph # line: 203
from tensorflow.python.framework.composite_tensor import CompositeTensor # line: 24
from tensorflow.python.framework.func_graph import FuncGraph # line: 133
from tensorflow.python.framework.ops import EagerTensor # line: 674
from tensorflow.python.framework.ops import SymbolicTensor # line: 259
from tensorflow.python.framework.ops import _create_c_op as create_c_op # line: 1020
from tensorflow.python.framework.ops import get_name_scope # line: 5176
from tensorflow.python.ops.control_flow_ops import get_enclosing_xla_context # line: 2208
from tensorflow.python.ops.control_flow_switch_case import execute_fn_for_device # line: 132
from tensorflow.python.util.keras_deps import register_call_context_function # line: 38
from tensorflow.python.util.keras_deps import register_clear_session_function # line: 44
from tensorflow.python.util.keras_deps import register_get_session_function # line: 50
from tensorflow.python.util.keras_deps import register_load_model_function # line: 56
