# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.linalg namespace
"""

import sys as _sys

from tensorflow._api.v2.compat.v1.linalg import experimental
from tensorflow.python.ops.gen_array_ops import matrix_band_part as band_part # line: 4921
from tensorflow.python.ops.gen_array_ops import diag as tensor_diag # line: 1949
from tensorflow.python.ops.gen_linalg_ops import cholesky # line: 766
from tensorflow.python.ops.gen_linalg_ops import matrix_determinant as det # line: 1370
from tensorflow.python.ops.gen_linalg_ops import matrix_inverse as inv # line: 1516
from tensorflow.python.ops.gen_linalg_ops import lu # line: 1252
from tensorflow.python.ops.gen_linalg_ops import qr # line: 2150
from tensorflow.python.ops.gen_linalg_ops import matrix_solve as solve # line: 1694
from tensorflow.python.ops.gen_linalg_ops import matrix_square_root as sqrtm # line: 1913
from tensorflow.python.ops.gen_math_ops import cross # line: 2708
from tensorflow.python.ops.array_ops import matrix_diag as diag # line: 2075
from tensorflow.python.ops.array_ops import matrix_diag_part as diag_part # line: 2244
from tensorflow.python.ops.array_ops import matrix_transpose # line: 1995
from tensorflow.python.ops.array_ops import matrix_set_diag as set_diag # line: 2432
from tensorflow.python.ops.array_ops import tensor_diag_part # line: 2387
from tensorflow.python.ops.array_ops import matrix_transpose as transpose # line: 1995
from tensorflow.python.ops.clip_ops import global_norm # line: 245
from tensorflow.python.ops.linalg.linalg_impl import adjoint # line: 102
from tensorflow.python.ops.linalg.linalg_impl import eigh_tridiagonal # line: 1232
from tensorflow.python.ops.linalg.linalg_impl import matrix_exponential as expm # line: 232
from tensorflow.python.ops.linalg.linalg_impl import logdet # line: 68
from tensorflow.python.ops.linalg.linalg_impl import logm # line: 54
from tensorflow.python.ops.linalg.linalg_impl import lu_matrix_inverse # line: 1035
from tensorflow.python.ops.linalg.linalg_impl import lu_reconstruct # line: 1100
from tensorflow.python.ops.linalg.linalg_impl import lu_solve # line: 937
from tensorflow.python.ops.linalg.linalg_impl import matrix_rank # line: 768
from tensorflow.python.ops.linalg.linalg_impl import pinv # line: 807
from tensorflow.python.ops.linalg.linalg_impl import slogdet # line: 44
from tensorflow.python.ops.linalg.linalg_impl import tridiagonal_matmul # line: 670
from tensorflow.python.ops.linalg.linalg_impl import tridiagonal_solve # line: 446
from tensorflow.python.ops.linalg.linear_operator import LinearOperator # line: 91
from tensorflow.python.ops.linalg.linear_operator_adjoint import LinearOperatorAdjoint # line: 28
from tensorflow.python.ops.linalg.linear_operator_block_diag import LinearOperatorBlockDiag # line: 34
from tensorflow.python.ops.linalg.linear_operator_block_lower_triangular import LinearOperatorBlockLowerTriangular # line: 39
from tensorflow.python.ops.linalg.linear_operator_circulant import LinearOperatorCirculant # line: 740
from tensorflow.python.ops.linalg.linear_operator_circulant import LinearOperatorCirculant2D # line: 1061
from tensorflow.python.ops.linalg.linear_operator_circulant import LinearOperatorCirculant3D # line: 1316
from tensorflow.python.ops.linalg.linear_operator_composition import LinearOperatorComposition # line: 35
from tensorflow.python.ops.linalg.linear_operator_diag import LinearOperatorDiag # line: 32
from tensorflow.python.ops.linalg.linear_operator_full_matrix import LinearOperatorFullMatrix # line: 29
from tensorflow.python.ops.linalg.linear_operator_householder import LinearOperatorHouseholder # line: 32
from tensorflow.python.ops.linalg.linear_operator_identity import LinearOperatorIdentity # line: 101
from tensorflow.python.ops.linalg.linear_operator_identity import LinearOperatorScaledIdentity # line: 538
from tensorflow.python.ops.linalg.linear_operator_inversion import LinearOperatorInversion # line: 25
from tensorflow.python.ops.linalg.linear_operator_kronecker import LinearOperatorKronecker # line: 60
from tensorflow.python.ops.linalg.linear_operator_low_rank_update import LinearOperatorLowRankUpdate # line: 35
from tensorflow.python.ops.linalg.linear_operator_lower_triangular import LinearOperatorLowerTriangular # line: 31
from tensorflow.python.ops.linalg.linear_operator_permutation import LinearOperatorPermutation # line: 35
from tensorflow.python.ops.linalg.linear_operator_toeplitz import LinearOperatorToeplitz # line: 33
from tensorflow.python.ops.linalg.linear_operator_tridiag import LinearOperatorTridiag # line: 39
from tensorflow.python.ops.linalg.linear_operator_zeros import LinearOperatorZeros # line: 39
from tensorflow.python.ops.linalg_ops import cholesky_solve # line: 147
from tensorflow.python.ops.linalg_ops import self_adjoint_eig as eigh # line: 441
from tensorflow.python.ops.linalg_ops import self_adjoint_eigvals as eigvalsh # line: 465
from tensorflow.python.ops.linalg_ops import eye # line: 196
from tensorflow.python.ops.linalg_ops import matrix_solve_ls as lstsq # line: 244
from tensorflow.python.ops.linalg_ops import norm # line: 633
from tensorflow.python.ops.linalg_ops import svd # line: 489
from tensorflow.python.ops.linalg_ops import matrix_triangular_solve as triangular_solve # line: 84
from tensorflow.python.ops.math_ops import matmul # line: 3443
from tensorflow.python.ops.math_ops import matvec # line: 3711
from tensorflow.python.ops.math_ops import tensordot # line: 5215
from tensorflow.python.ops.math_ops import trace # line: 3399
from tensorflow.python.ops.nn_impl import l2_normalize # line: 540
from tensorflow.python.ops.nn_impl import normalize # line: 487
from tensorflow.python.ops.special_math_ops import einsum # line: 618

from tensorflow.python.util import module_wrapper as _module_wrapper

if not isinstance(_sys.modules[__name__], _module_wrapper.TFModuleWrapper):
  _sys.modules[__name__] = _module_wrapper.TFModuleWrapper(
      _sys.modules[__name__], "linalg", public_apis=None, deprecation=False,
      has_lite=False)
