// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: tensorflow/core/protobuf/device_filters.proto
// Protobuf C++ Version: 5.28.3

#ifndef GOOGLE_PROTOBUF_INCLUDED_tensorflow_2fcore_2fprotobuf_2fdevice_5ffilters_2eproto_2epb_2eh
#define GOOGLE_PROTOBUF_INCLUDED_tensorflow_2fcore_2fprotobuf_2fdevice_5ffilters_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 5028003
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/map.h"  // IWYU pragma: export
#include "google/protobuf/map_entry.h"
#include "google/protobuf/map_field_inl.h"
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_tensorflow_2fcore_2fprotobuf_2fdevice_5ffilters_2eproto PROTOBUF_EXPORT

namespace google {
namespace protobuf {
namespace internal {
class AnyMetadata;
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct PROTOBUF_EXPORT TableStruct_tensorflow_2fcore_2fprotobuf_2fdevice_5ffilters_2eproto {
  static const ::uint32_t offsets[];
};
PROTOBUF_EXPORT extern const ::google::protobuf::internal::DescriptorTable
    descriptor_table_tensorflow_2fcore_2fprotobuf_2fdevice_5ffilters_2eproto;
namespace tensorflow {
class ClusterDeviceFilters;
struct ClusterDeviceFiltersDefaultTypeInternal;
PROTOBUF_EXPORT extern ClusterDeviceFiltersDefaultTypeInternal _ClusterDeviceFilters_default_instance_;
class JobDeviceFilters;
struct JobDeviceFiltersDefaultTypeInternal;
PROTOBUF_EXPORT extern JobDeviceFiltersDefaultTypeInternal _JobDeviceFilters_default_instance_;
class JobDeviceFilters_TasksEntry_DoNotUse;
struct JobDeviceFilters_TasksEntry_DoNotUseDefaultTypeInternal;
PROTOBUF_EXPORT extern JobDeviceFilters_TasksEntry_DoNotUseDefaultTypeInternal _JobDeviceFilters_TasksEntry_DoNotUse_default_instance_;
class TaskDeviceFilters;
struct TaskDeviceFiltersDefaultTypeInternal;
PROTOBUF_EXPORT extern TaskDeviceFiltersDefaultTypeInternal _TaskDeviceFilters_default_instance_;
}  // namespace tensorflow
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace tensorflow {

// ===================================================================


// -------------------------------------------------------------------

class PROTOBUF_EXPORT TaskDeviceFilters final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:tensorflow.TaskDeviceFilters) */ {
 public:
  inline TaskDeviceFilters() : TaskDeviceFilters(nullptr) {}
  ~TaskDeviceFilters() PROTOBUF_FINAL;
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR TaskDeviceFilters(
      ::google::protobuf::internal::ConstantInitialized);

  inline TaskDeviceFilters(const TaskDeviceFilters& from) : TaskDeviceFilters(nullptr, from) {}
  inline TaskDeviceFilters(TaskDeviceFilters&& from) noexcept
      : TaskDeviceFilters(nullptr, std::move(from)) {}
  inline TaskDeviceFilters& operator=(const TaskDeviceFilters& from) {
    CopyFrom(from);
    return *this;
  }
  inline TaskDeviceFilters& operator=(TaskDeviceFilters&& from) noexcept {
    if (this == &from) return *this;
    if (GetArena() == from.GetArena()
#ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetArena() != nullptr
#endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const TaskDeviceFilters& default_instance() {
    return *internal_default_instance();
  }
  static inline const TaskDeviceFilters* internal_default_instance() {
    return reinterpret_cast<const TaskDeviceFilters*>(
        &_TaskDeviceFilters_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(TaskDeviceFilters& a, TaskDeviceFilters& b) { a.Swap(&b); }
  inline void Swap(TaskDeviceFilters* other) {
    if (other == this) return;
#ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() != nullptr && GetArena() == other->GetArena()) {
#else   // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() == other->GetArena()) {
#endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TaskDeviceFilters* other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  TaskDeviceFilters* New(::google::protobuf::Arena* arena = nullptr) const PROTOBUF_FINAL {
    return ::google::protobuf::Message::DefaultConstruct<TaskDeviceFilters>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const TaskDeviceFilters& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const TaskDeviceFilters& from) { TaskDeviceFilters::MergeImpl(*this, from); }

  private:
  static void MergeImpl(
      ::google::protobuf::MessageLite& to_msg,
      const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* _InternalSerialize(
      const MessageLite& msg, ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* arena);
  void SharedDtor();
  void InternalSwap(TaskDeviceFilters* other);
 private:
  friend class ::google::protobuf::internal::AnyMetadata;
  static ::absl::string_view FullMessageName() { return "tensorflow.TaskDeviceFilters"; }

 protected:
  explicit TaskDeviceFilters(::google::protobuf::Arena* arena);
  TaskDeviceFilters(::google::protobuf::Arena* arena, const TaskDeviceFilters& from);
  TaskDeviceFilters(::google::protobuf::Arena* arena, TaskDeviceFilters&& from) noexcept
      : TaskDeviceFilters(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;

 public:
  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kDeviceFiltersFieldNumber = 1,
  };
  // repeated string device_filters = 1;
  int device_filters_size() const;
  private:
  int _internal_device_filters_size() const;

  public:
  void clear_device_filters() ;
  const std::string& device_filters(int index) const;
  std::string* mutable_device_filters(int index);
  template <typename Arg_ = const std::string&, typename... Args_>
  void set_device_filters(int index, Arg_&& value, Args_... args);
  std::string* add_device_filters();
  template <typename Arg_ = const std::string&, typename... Args_>
  void add_device_filters(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<std::string>& device_filters() const;
  ::google::protobuf::RepeatedPtrField<std::string>* mutable_device_filters();

  private:
  const ::google::protobuf::RepeatedPtrField<std::string>& _internal_device_filters() const;
  ::google::protobuf::RepeatedPtrField<std::string>* _internal_mutable_device_filters();

  public:
  // @@protoc_insertion_point(class_scope:tensorflow.TaskDeviceFilters)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      0, 1, 0,
      51, 2>
      _table_;


  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(
        ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena);
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena, const Impl_& from,
                          const TaskDeviceFilters& from_msg);
    ::google::protobuf::RepeatedPtrField<std::string> device_filters_;
    mutable ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_tensorflow_2fcore_2fprotobuf_2fdevice_5ffilters_2eproto;
};
// -------------------------------------------------------------------

class JobDeviceFilters_TasksEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<
          JobDeviceFilters_TasksEntry_DoNotUse, ::int32_t, ::tensorflow::TaskDeviceFilters,
          ::google::protobuf::internal::WireFormatLite::TYPE_INT32,
          ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE> {
 public:
  using SuperType = ::google::protobuf::internal::MapEntry<
      JobDeviceFilters_TasksEntry_DoNotUse, ::int32_t, ::tensorflow::TaskDeviceFilters,
      ::google::protobuf::internal::WireFormatLite::TYPE_INT32,
      ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE>;
  JobDeviceFilters_TasksEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR JobDeviceFilters_TasksEntry_DoNotUse(
      ::google::protobuf::internal::ConstantInitialized);
  explicit JobDeviceFilters_TasksEntry_DoNotUse(::google::protobuf::Arena* arena);
  static const JobDeviceFilters_TasksEntry_DoNotUse* internal_default_instance() {
    return reinterpret_cast<const JobDeviceFilters_TasksEntry_DoNotUse*>(
        &_JobDeviceFilters_TasksEntry_DoNotUse_default_instance_);
  }


 private:
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      1, 2, 1,
      0, 2>
      _table_;

  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;
  friend struct ::TableStruct_tensorflow_2fcore_2fprotobuf_2fdevice_5ffilters_2eproto;
};
// -------------------------------------------------------------------

class PROTOBUF_EXPORT JobDeviceFilters final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:tensorflow.JobDeviceFilters) */ {
 public:
  inline JobDeviceFilters() : JobDeviceFilters(nullptr) {}
  ~JobDeviceFilters() PROTOBUF_FINAL;
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR JobDeviceFilters(
      ::google::protobuf::internal::ConstantInitialized);

  inline JobDeviceFilters(const JobDeviceFilters& from) : JobDeviceFilters(nullptr, from) {}
  inline JobDeviceFilters(JobDeviceFilters&& from) noexcept
      : JobDeviceFilters(nullptr, std::move(from)) {}
  inline JobDeviceFilters& operator=(const JobDeviceFilters& from) {
    CopyFrom(from);
    return *this;
  }
  inline JobDeviceFilters& operator=(JobDeviceFilters&& from) noexcept {
    if (this == &from) return *this;
    if (GetArena() == from.GetArena()
#ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetArena() != nullptr
#endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const JobDeviceFilters& default_instance() {
    return *internal_default_instance();
  }
  static inline const JobDeviceFilters* internal_default_instance() {
    return reinterpret_cast<const JobDeviceFilters*>(
        &_JobDeviceFilters_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(JobDeviceFilters& a, JobDeviceFilters& b) { a.Swap(&b); }
  inline void Swap(JobDeviceFilters* other) {
    if (other == this) return;
#ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() != nullptr && GetArena() == other->GetArena()) {
#else   // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() == other->GetArena()) {
#endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(JobDeviceFilters* other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  JobDeviceFilters* New(::google::protobuf::Arena* arena = nullptr) const PROTOBUF_FINAL {
    return ::google::protobuf::Message::DefaultConstruct<JobDeviceFilters>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const JobDeviceFilters& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const JobDeviceFilters& from) { JobDeviceFilters::MergeImpl(*this, from); }

  private:
  static void MergeImpl(
      ::google::protobuf::MessageLite& to_msg,
      const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* _InternalSerialize(
      const MessageLite& msg, ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* arena);
  void SharedDtor();
  void InternalSwap(JobDeviceFilters* other);
 private:
  friend class ::google::protobuf::internal::AnyMetadata;
  static ::absl::string_view FullMessageName() { return "tensorflow.JobDeviceFilters"; }

 protected:
  explicit JobDeviceFilters(::google::protobuf::Arena* arena);
  JobDeviceFilters(::google::protobuf::Arena* arena, const JobDeviceFilters& from);
  JobDeviceFilters(::google::protobuf::Arena* arena, JobDeviceFilters&& from) noexcept
      : JobDeviceFilters(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;

 public:
  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kTasksFieldNumber = 2,
    kNameFieldNumber = 1,
  };
  // map<int32, .tensorflow.TaskDeviceFilters> tasks = 2;
  int tasks_size() const;
  private:
  int _internal_tasks_size() const;

  public:
  void clear_tasks() ;
  const ::google::protobuf::Map<::int32_t, ::tensorflow::TaskDeviceFilters>& tasks() const;
  ::google::protobuf::Map<::int32_t, ::tensorflow::TaskDeviceFilters>* mutable_tasks();

  private:
  const ::google::protobuf::Map<::int32_t, ::tensorflow::TaskDeviceFilters>& _internal_tasks() const;
  ::google::protobuf::Map<::int32_t, ::tensorflow::TaskDeviceFilters>* _internal_mutable_tasks();

  public:
  // string name = 1;
  void clear_name() ;
  const std::string& name() const;
  template <typename Arg_ = const std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  std::string* mutable_name();
  PROTOBUF_NODISCARD std::string* release_name();
  void set_allocated_name(std::string* value);

  private:
  const std::string& _internal_name() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_name(
      const std::string& value);
  std::string* _internal_mutable_name();

  public:
  // @@protoc_insertion_point(class_scope:tensorflow.JobDeviceFilters)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      0, 2, 2,
      40, 2>
      _table_;


  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(
        ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena);
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena, const Impl_& from,
                          const JobDeviceFilters& from_msg);
    ::google::protobuf::internal::MapField<JobDeviceFilters_TasksEntry_DoNotUse, ::int32_t, ::tensorflow::TaskDeviceFilters,
                      ::google::protobuf::internal::WireFormatLite::TYPE_INT32,
                      ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE>
        tasks_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    mutable ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_tensorflow_2fcore_2fprotobuf_2fdevice_5ffilters_2eproto;
};
// -------------------------------------------------------------------

class PROTOBUF_EXPORT ClusterDeviceFilters final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:tensorflow.ClusterDeviceFilters) */ {
 public:
  inline ClusterDeviceFilters() : ClusterDeviceFilters(nullptr) {}
  ~ClusterDeviceFilters() PROTOBUF_FINAL;
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ClusterDeviceFilters(
      ::google::protobuf::internal::ConstantInitialized);

  inline ClusterDeviceFilters(const ClusterDeviceFilters& from) : ClusterDeviceFilters(nullptr, from) {}
  inline ClusterDeviceFilters(ClusterDeviceFilters&& from) noexcept
      : ClusterDeviceFilters(nullptr, std::move(from)) {}
  inline ClusterDeviceFilters& operator=(const ClusterDeviceFilters& from) {
    CopyFrom(from);
    return *this;
  }
  inline ClusterDeviceFilters& operator=(ClusterDeviceFilters&& from) noexcept {
    if (this == &from) return *this;
    if (GetArena() == from.GetArena()
#ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetArena() != nullptr
#endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ClusterDeviceFilters& default_instance() {
    return *internal_default_instance();
  }
  static inline const ClusterDeviceFilters* internal_default_instance() {
    return reinterpret_cast<const ClusterDeviceFilters*>(
        &_ClusterDeviceFilters_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(ClusterDeviceFilters& a, ClusterDeviceFilters& b) { a.Swap(&b); }
  inline void Swap(ClusterDeviceFilters* other) {
    if (other == this) return;
#ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() != nullptr && GetArena() == other->GetArena()) {
#else   // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetArena() == other->GetArena()) {
#endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ClusterDeviceFilters* other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ClusterDeviceFilters* New(::google::protobuf::Arena* arena = nullptr) const PROTOBUF_FINAL {
    return ::google::protobuf::Message::DefaultConstruct<ClusterDeviceFilters>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ClusterDeviceFilters& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ClusterDeviceFilters& from) { ClusterDeviceFilters::MergeImpl(*this, from); }

  private:
  static void MergeImpl(
      ::google::protobuf::MessageLite& to_msg,
      const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* _InternalSerialize(
      const MessageLite& msg, ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* _InternalSerialize(
      ::uint8_t* target,
      ::google::protobuf::io::EpsCopyOutputStream* stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* arena);
  void SharedDtor();
  void InternalSwap(ClusterDeviceFilters* other);
 private:
  friend class ::google::protobuf::internal::AnyMetadata;
  static ::absl::string_view FullMessageName() { return "tensorflow.ClusterDeviceFilters"; }

 protected:
  explicit ClusterDeviceFilters(::google::protobuf::Arena* arena);
  ClusterDeviceFilters(::google::protobuf::Arena* arena, const ClusterDeviceFilters& from);
  ClusterDeviceFilters(::google::protobuf::Arena* arena, ClusterDeviceFilters&& from) noexcept
      : ClusterDeviceFilters(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::Message::ClassData* GetClassData() const PROTOBUF_FINAL;
  static const ::google::protobuf::Message::ClassDataFull _class_data_;

 public:
  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kJobsFieldNumber = 1,
  };
  // repeated .tensorflow.JobDeviceFilters jobs = 1;
  int jobs_size() const;
  private:
  int _internal_jobs_size() const;

  public:
  void clear_jobs() ;
  ::tensorflow::JobDeviceFilters* mutable_jobs(int index);
  ::google::protobuf::RepeatedPtrField<::tensorflow::JobDeviceFilters>* mutable_jobs();

  private:
  const ::google::protobuf::RepeatedPtrField<::tensorflow::JobDeviceFilters>& _internal_jobs() const;
  ::google::protobuf::RepeatedPtrField<::tensorflow::JobDeviceFilters>* _internal_mutable_jobs();
  public:
  const ::tensorflow::JobDeviceFilters& jobs(int index) const;
  ::tensorflow::JobDeviceFilters* add_jobs();
  const ::google::protobuf::RepeatedPtrField<::tensorflow::JobDeviceFilters>& jobs() const;
  // @@protoc_insertion_point(class_scope:tensorflow.ClusterDeviceFilters)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<
      0, 1, 1,
      0, 2>
      _table_;


  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(
        ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena);
    inline explicit Impl_(::google::protobuf::internal::InternalVisibility visibility,
                          ::google::protobuf::Arena* arena, const Impl_& from,
                          const ClusterDeviceFilters& from_msg);
    ::google::protobuf::RepeatedPtrField< ::tensorflow::JobDeviceFilters > jobs_;
    mutable ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_tensorflow_2fcore_2fprotobuf_2fdevice_5ffilters_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// TaskDeviceFilters

// repeated string device_filters = 1;
inline int TaskDeviceFilters::_internal_device_filters_size() const {
  return _internal_device_filters().size();
}
inline int TaskDeviceFilters::device_filters_size() const {
  return _internal_device_filters_size();
}
inline void TaskDeviceFilters::clear_device_filters() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.device_filters_.Clear();
}
inline std::string* TaskDeviceFilters::add_device_filters() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  std::string* _s = _internal_mutable_device_filters()->Add();
  // @@protoc_insertion_point(field_add_mutable:tensorflow.TaskDeviceFilters.device_filters)
  return _s;
}
inline const std::string& TaskDeviceFilters::device_filters(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.TaskDeviceFilters.device_filters)
  return _internal_device_filters().Get(index);
}
inline std::string* TaskDeviceFilters::mutable_device_filters(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:tensorflow.TaskDeviceFilters.device_filters)
  return _internal_mutable_device_filters()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void TaskDeviceFilters::set_device_filters(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_device_filters()->Mutable(index),
      std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:tensorflow.TaskDeviceFilters.device_filters)
}
template <typename Arg_, typename... Args_>
inline void TaskDeviceFilters::add_device_filters(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(*_internal_mutable_device_filters(),
                               std::forward<Arg_>(value),
                               args... );
  // @@protoc_insertion_point(field_add:tensorflow.TaskDeviceFilters.device_filters)
}
inline const ::google::protobuf::RepeatedPtrField<std::string>&
TaskDeviceFilters::device_filters() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:tensorflow.TaskDeviceFilters.device_filters)
  return _internal_device_filters();
}
inline ::google::protobuf::RepeatedPtrField<std::string>*
TaskDeviceFilters::mutable_device_filters() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:tensorflow.TaskDeviceFilters.device_filters)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_device_filters();
}
inline const ::google::protobuf::RepeatedPtrField<std::string>&
TaskDeviceFilters::_internal_device_filters() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.device_filters_;
}
inline ::google::protobuf::RepeatedPtrField<std::string>*
TaskDeviceFilters::_internal_mutable_device_filters() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.device_filters_;
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// JobDeviceFilters

// string name = 1;
inline void JobDeviceFilters::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
}
inline const std::string& JobDeviceFilters::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.JobDeviceFilters.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
inline PROTOBUF_ALWAYS_INLINE void JobDeviceFilters::set_name(Arg_&& arg,
                                                     Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:tensorflow.JobDeviceFilters.name)
}
inline std::string* JobDeviceFilters::mutable_name() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:tensorflow.JobDeviceFilters.name)
  return _s;
}
inline const std::string& JobDeviceFilters::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void JobDeviceFilters::_internal_set_name(const std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline std::string* JobDeviceFilters::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline std::string* JobDeviceFilters::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:tensorflow.JobDeviceFilters.name)
  return _impl_.name_.Release();
}
inline void JobDeviceFilters::set_allocated_name(std::string* value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.SetAllocated(value, GetArena());
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
        if (_impl_.name_.IsDefault()) {
          _impl_.name_.Set("", GetArena());
        }
  #endif  // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:tensorflow.JobDeviceFilters.name)
}

// map<int32, .tensorflow.TaskDeviceFilters> tasks = 2;
inline int JobDeviceFilters::_internal_tasks_size() const {
  return _internal_tasks().size();
}
inline int JobDeviceFilters::tasks_size() const {
  return _internal_tasks_size();
}
inline void JobDeviceFilters::clear_tasks() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.tasks_.Clear();
}
inline const ::google::protobuf::Map<::int32_t, ::tensorflow::TaskDeviceFilters>& JobDeviceFilters::_internal_tasks() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.tasks_.GetMap();
}
inline const ::google::protobuf::Map<::int32_t, ::tensorflow::TaskDeviceFilters>& JobDeviceFilters::tasks() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:tensorflow.JobDeviceFilters.tasks)
  return _internal_tasks();
}
inline ::google::protobuf::Map<::int32_t, ::tensorflow::TaskDeviceFilters>* JobDeviceFilters::_internal_mutable_tasks() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.tasks_.MutableMap();
}
inline ::google::protobuf::Map<::int32_t, ::tensorflow::TaskDeviceFilters>* JobDeviceFilters::mutable_tasks() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_map:tensorflow.JobDeviceFilters.tasks)
  return _internal_mutable_tasks();
}

// -------------------------------------------------------------------

// ClusterDeviceFilters

// repeated .tensorflow.JobDeviceFilters jobs = 1;
inline int ClusterDeviceFilters::_internal_jobs_size() const {
  return _internal_jobs().size();
}
inline int ClusterDeviceFilters::jobs_size() const {
  return _internal_jobs_size();
}
inline void ClusterDeviceFilters::clear_jobs() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.jobs_.Clear();
}
inline ::tensorflow::JobDeviceFilters* ClusterDeviceFilters::mutable_jobs(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:tensorflow.ClusterDeviceFilters.jobs)
  return _internal_mutable_jobs()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::tensorflow::JobDeviceFilters>* ClusterDeviceFilters::mutable_jobs()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:tensorflow.ClusterDeviceFilters.jobs)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_jobs();
}
inline const ::tensorflow::JobDeviceFilters& ClusterDeviceFilters::jobs(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.ClusterDeviceFilters.jobs)
  return _internal_jobs().Get(index);
}
inline ::tensorflow::JobDeviceFilters* ClusterDeviceFilters::add_jobs() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::tensorflow::JobDeviceFilters* _add = _internal_mutable_jobs()->Add();
  // @@protoc_insertion_point(field_add:tensorflow.ClusterDeviceFilters.jobs)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::tensorflow::JobDeviceFilters>& ClusterDeviceFilters::jobs() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:tensorflow.ClusterDeviceFilters.jobs)
  return _internal_jobs();
}
inline const ::google::protobuf::RepeatedPtrField<::tensorflow::JobDeviceFilters>&
ClusterDeviceFilters::_internal_jobs() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.jobs_;
}
inline ::google::protobuf::RepeatedPtrField<::tensorflow::JobDeviceFilters>*
ClusterDeviceFilters::_internal_mutable_jobs() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.jobs_;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace tensorflow


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // GOOGLE_PROTOBUF_INCLUDED_tensorflow_2fcore_2fprotobuf_2fdevice_5ffilters_2eproto_2epb_2eh
