/* Copyright 2024 The Shardy Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================*/

#ifndef SHARDY_DIALECT_SDY_TRANSFORMS_COMMON_OP_PROPERTIES_H_
#define SHARDY_DIALECT_SDY_TRANSFORMS_COMMON_OP_PROPERTIES_H_

#include "mlir/IR/Operation.h"

namespace mlir {
namespace sdy {

// Returns true if the given `op` is elementwise on tensor operands and results.
//
// An op is element-wise iff every element in the result tensor depends only on
// the element in the input tensors with the same index.
bool isElementwise(Operation* op);

}  // namespace sdy
}  // namespace mlir

#endif  // SHARDY_DIALECT_SDY_TRANSFORMS_COMMON_OP_PROPERTIES_H_
