/* Copyright 2025 The Shardy Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================*/

#ifndef SHARDY_DIALECT_SDY_IR_EXTENSIONS_STABLEHLO_EXTENSIONS_H_
#define SHARDY_DIALECT_SDY_IR_EXTENSIONS_STABLEHLO_EXTENSIONS_H_

#include "mlir/IR/MLIRContext.h"

namespace mlir {
namespace sdy {

// Register Shardy op interface extensions to StableHLO ops.
void registerStablehloExtensions(MLIRContext *ctx);

}  // namespace sdy
}  // namespace mlir

#endif  // SHARDY_DIALECT_SDY_IR_EXTENSIONS_STABLEHLO_EXTENSIONS_H_
