# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.__internal__.nest namespace
"""

import sys as _sys

from tensorflow.python.util.nest import flatten_up_to # line: 764
from tensorflow.python.util.nest import get_traverse_shallow_structure # line: 1115
from tensorflow.python.util.nest import is_attrs # line: 138
from tensorflow.python.util.nest import is_mapping # line: 131
from tensorflow.python.util.nest import list_to_tuple # line: 1288
from tensorflow.python.util.nest import map_structure_up_to # line: 960
from tensorflow.python.util.nest import _sequence_like as sequence_like # line: 144
from tensorflow.python.util.nest import yield_flat_paths # line: 1188
